; -----------------------------------------------------------------------------
; CHETIRY (GameBoyTM Version) - Atari 2600 - (C) Copyright 2011 - AtariAge
; BANK 1 - MAIN GAME
; -----------------------------------------------------------------------------

; Flash Masks
FLASHON                   = %01000000
FLASHOFF                  = %00000000
  
; GameOver Fill Colours
FILL0                     = %10111111     ; Fill A (BCOL7)
FILL1                     = %11111111     ; Fill B (BCOL7)

  ; Load Temporary Shape Values
  MAC LOADTEMPS
  ; Y Position
  lax YPOS                ; [0] + 3
  and #%00011111          ; [3] + 2
  sta TEMPY               ; [5] + 3
  ; Rotation
  txa                     ; [8] + 2
  and #%01100000          ; [10] + 2
  sta TEMPROT             ; [12] + 3
  ; Rotation * 4
  lsr                     ; [15] + 2
  lsr                     ; [17] + 2
  lsr                     ; [19] + 2
  sta TEMP                ; [21] + 3
  ; X Position (-1)
  lax XPOS                ; [24] + 3
  and #%00001111          ; [27] + 2
  sec                     ; [29] + 2
  sbc #1                  ; [31] + 2
  sta TEMPX               ; [33] + 3
  ; Shape Type
  txa                     ; [36] + 2
  and #%01110000          ; [38] + 2
  sta TEMPVAL             ; [40] + 3
  ; Shape Pointer (Type * 32 + Rotation * 4)
  asl                     ; [43] + 2
  ora TEMP                ; [45] + 3
  sta SPTR                ; [48] + 3      = 51
  ENDM

; -----------------------------------------------------------------------------

  MAC SHAPEVALS
  ; Calculate Shifted Shape Values
  lax TEMPVAL             ; [0] + 3
  lsr                     ; [3] + 2
  sta VALUE2              ; [5] + 3
  lsr                     ; [8] + 2
  lsr                     ; [10] + 2
  lsr                     ; [12] + 2
  sta VALUE1              ; [14] + 3
  txa                     ; [17] + 2
  rol                     ; [19] + 2
  rol                     ; [21] + 2
  sta VALUE4              ; [23] + 3
  lda #0                  ; [26] + 2
  ror                     ; [28] + 2
  sta VALUE3              ; [30] + 3      = 33
  ENDM
  
; -----------------------------------------------------------------------------

  ; Set/Reset Drop Timer
  MAC SETTIMER
  ; Store Next Piece
  lda TIMER               ; [0] + 3
  and #%11100000          ; [3] + 2
  sta TEMP                ; [5] + 3
  ; Get Level
  lda LEVEL               ; [8] + 3
  and #%00011111          ; [11] + 2
  tay                     ; [13] + 2
  ; Lightning After Level 20
  lda #2                  ; [15] + 2
  cpy #19                 ; [17] + 2
  bcc .NormalTimer        ; [19] + 2/3
  nop                     ; [21] + 2
  bcs .StoreTimer         ; [23] + 3
.NormalTimer
  ; Normal Levels (0->19)
  lda Levels,Y            ; [22] + 4
.StoreTimer
  ora TEMP                ; [26] + 3
  sta TIMER               ; [29] + 3      = 32
  ENDM

; -----------------------------------------------------------------------------

; Get Data at Grid Position (bottom left = 0,0)
; X = Row (0-17)
; A = Column (0-9)
  MAC GETDATA
.ENDJUMP SET {1}
  ; Check if column is off the edge (error)
  cmp #10                 ; [0] + 2
  bcs .GetError           ; [2] + 2/3
  ; Check if row is off the top/bottom of grid
  cpx #18                 ; [4] + 2
  bcs .GetTop             ; [6] + 2/3
  ; Start position = X * 4
  ldy StartPos,X          ; [8] + 4
  ; Special case for position 8 & 9
  cmp #8                  ; [12] + 2
  bcc .GetNormal          ; [14] + 2/3
  beq .GetEight           ; [16] + 2/3
.GetNine
  ; Fetch data
  lda GRID+2,Y            ; [18] + 4
  rol                     ; [22] + 2
  ; Merge data 
  lda GRID+3,Y            ; [24] + 4
  rol                     ; [28] + 2
  rol                     ; [30] + 2
  rol                     ; [32] + 2
  and #%00000111          ; [34] + 2
  jmp .ENDJUMP            ; [36] + 3      = 39
.GetEight
  ; Fetch data
  lda GRID+0,Y            ; [19] + 4
  rol                     ; [23] + 2
  ; Merge data 
  lda GRID+1,Y            ; [25] + 4
  rol                     ; [29] + 2
  rol                     ; [31] + 2
  rol                     ; [33] + 2
  and #%00000111          ; [35] + 2
  jmp .ENDJUMP            ; [37] + 3      = 40
.GetTop
  ; Check if row is off bottom of grid (error)
  txa                     ; [9] + 2
  bmi .GetError           ; [11] + 2/3
  lda #0                  ; [13] + 2
  jmp .ENDJUMP            ; [15] + 3      = 18
.GetError
  lda #%00000111          ; [14] + 2
  jmp .ENDJUMP            ; [16] + 3      = 19
.GetNormal
  ; Start at A/2
  sty TEMP                ; [17] + 3
  lsr                     ; [20] + 2
  bcs .GetUpper           ; [22] + 2/3
.GetLower
  ; Start position + Y
  ; clc
  adc TEMP                ; [24] + 3
  tay                     ; [27] + 2
  lda GRID,Y              ; [29] + 4
  and #%00000111          ; [33] + 2
  jmp .ENDJUMP            ; [35] + 3      = 38
.GetUpper
  ; Start position + Y
  clc                     ; [25] + 2
  adc TEMP                ; [27] + 3
  tay                     ; [30] + 2
  lda GRID,Y              ; [32] + 4
  and #%00111000          ; [36] + 2      = 38
  ENDM

; -----------------------------------------------------------------------------

; Draw Current Shape on Grid
; X = Row (0-17)
; A = Column (0-9) 
; VALUE1-4 = Shape Colour (0-7)
  MAC SETDATA
.ENDJUMP    SET {1}
.SKIPLABEL  SET {2}
  ; Check if column is off the edge (ignore)
  cmp #10                 ; [0] + 2
  bcs .SKIPLABEL          ; [2] + 2/3
  ; Special case for position 8 & 9
  cmp #8                  ; [4] + 2
  bcc .SetNormal          ; [6] + 2/3
  beq .SetEight           ; [8] + 2/3
.SetNine
  ; Set position 9
  lda GRID+2,X            ; [10] + 4
  and #%00111111          ; [14] + 2
  ora VALUE3              ; [16] + 3
  sta GRID+2,X            ; [19] + 4
  lda GRID+3,X            ; [23] + 4
  and #%00111111          ; [27] + 2
  ora.w VALUE4            ; [29] + 4
  sta GRID+3,X            ; [33] + 4
  jmp .ENDJUMP            ; [37] + 3      = 40
.SetEight
  ; Set position 8
  lda GRID,X              ; [11] + 4
  and #%00111111          ; [15] + 2
  ora VALUE3              ; [17] + 3
  sta GRID,X              ; [20] + 4
  lda GRID+1,X            ; [24] + 4
  and #%00111111          ; [28] + 2
  ora VALUE4              ; [30] + 3
  sta GRID+1,X            ; [33] + 4
  jmp .ENDJUMP            ; [37] + 3      = 40
.SetLower
  ; Set lower position
  ; clc
  adc TEMP                ; [19] + 3
  tax                     ; [22] + 2
  lda GRID,X              ; [24] + 4
  and #%11111000          ; [28] + 2
  ora VALUE1              ; [30] + 3
  sta GRID,X              ; [33] + 4
  jmp .ENDJUMP            ; [37] + 3      = 40
.SetNormal
  ; Start at column/2
  stx TEMP                ; [9] + 3
  nop                     ; [12] + 2
  lsr                     ; [14] + 2
  bcc .SetLower           ; [16] + 2/3
.SetUpper
  ; Set upper position
  ; clc
  adc TEMP                ; [18] + 3
  tax                     ; [21] + 2
  lda GRID-1,X            ; [23] + 4
  and #%11000111          ; [27] + 2
  ora.w VALUE2            ; [29] + 4
  sta GRID-1,X            ; [33] + 4      = 37
  ENDM

; -----------------------------------------------------------------------------

; BANK 1
  SEG     BANK1
  ORG     $0000
  RORG    $F000

  ; Space for DPC registers
  DS.B    128,0

Init1
  ; Switch to Bank 2
  nop     $FFF9
  ; Begin Game
  jmp     GameStart
StartKernel2
  ; Switch To Bank 2
  nop     $FFF9
  ; Resume After Kernel
  jmp     OverLogic
FinishGame
  ; Switch To Bank 2
  nop     $FFF9
  
; -----------------------------------------------------------------------------
; GAME SETUP CODE
; -----------------------------------------------------------------------------

GameStart
  ; Reset Score
  ldy #0
  sty SCORE+0
  sty SCORE+1
  sty SCORE+2
  
  ; Reset Lines
  sty LINES+0
  sty LINES+1

  ; Reset Level
  sty LEVEL

  ; Clear Well
  ldx #71
ClearWellLoop
  sty GRID,X
  dex
  bpl ClearWellLoop

  ; Random Initial Shape
  jsr GetRandom
  ldy RANDOM
  lda Mod7,Y
  sta TIMER

  ; Spawn First Shape
  jsr GetRandom
  jmp Spawn

; -----------------------------------------------------------------------------
; MAIN GAME LOOP
; -----------------------------------------------------------------------------

MainLoop
  lda #2
  sta WSYNC               ; [0]
  sta VSYNC               ; [0] + 3       VSYNC enable
  PLAY_MUSIC              ; [3] + 7

  ; Get Y Position
  lax YPOS                ; [10] + 3
  and #%00011111          ; [13] + 2
  sta TEMPY               ; [15] + 3
  ; Get Rotation
  txa                     ; [18] + 2
  and #%01100000          ; [20] + 2
  sta TEMPROT             ; [22] + 3
  sta TEMPROT2            ; [25] + 3
  lsr                     ; [28] + 2
  lsr                     ; [30] + 2
  lsr                     ; [32] + 2
  sta TEMP                ; [34] + 3
  ; Get X Position
  lax XPOS                ; [37] + 3
  and #%00001111          ; [40] + 2
  sec                     ; [42] + 2
  sbc #1                  ; [44] + 2
  sta TEMPX               ; [46] + 3
  sta TEMPX2              ; [49] + 3
  ; Get Shape Type
  txa                     ; [52] + 2
  and #%01110000          ; [54] + 2
  sta TEMPVAL             ; [56] + 3
  ; Get Shape Pointer (Type * 32 + Rotation * 4)
  asl                     ; [59] + 2
  ora TEMP                ; [61] + 3
  sta SPTR                ; [64] + 3
  sta SPTR2               ; [67] + 3

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Calculate Shifted Shape Values
  SHAPEVALS               ; [7] + 33

  ; Update Random Number Generator
  jsr GetRandom           ; [40] + 24 + 6 = 70

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Check For Reset Switch
  lda SWCHB               ; [7] + 3
  lsr                     ; [10] + 2
  bcs NoReset             ; [12] + 2/3
  ; Reset Score
  lda #0                  ; [14] + 2
  sta SCORE+0             ; [16] + 3
  sta SCORE+1             ; [19] + 3
  sta SCORE+2             ; [22] + 3
  ; Start Frame
  sta WSYNC
  sta VSYNC
  lda #VBLANKDELAY
  sta TIM64T
  ; Quit Game
  jmp EndGame
NoReset

  ; Set VBlank Timer
  ldx #VBLANKDELAY
  lda #0
  sta WSYNC               ; [0]
  sta VSYNC               ; [0] + 3       VSYNC disable
  PLAY_MUSIC              ; [3] + 7
  stx TIM64T              ; [10] + 3

  ; Check Mode Bits
CheckMode
  bit LINES+1             ; [13] + 3
  bvs SettleMode1         ; [16] + 2/3    Settle Mode
  bmi FlashMode           ; [18] + 2/3    Flash Mode
  jmp NormalMode          ; [20] + 3      Normal Mode
SettleMode1
  bmi GameOverMode        ; [19] + 2/3    Game Over Mode
  jmp SettleMode          ; [21] + 3

GameOverMode
  ; Check Timer
  lax TIMER               ; [22] + 3
  and #%00011111          ; [25] + 2
  beq CheckGameOver       ; [27] + 2/3
  ; Check GameOver Type
  txa                     ; [29] + 2
  and #%11100000          ; [31] + 2
  beq GameOverFill        ; [33] + 2/3
  bne WaitGameEnd         ; [35] + 3
CheckGameOver
  ; Ensure Sounds Are Done
  lda SFX                 ; [30] + 3
  bne WaitGameEnd         ; [33] + 2/3
  ; Check Fire Button
  lda INPT4               ; [35] + 3
  bmi ResetGameOverFire   ; [38] + 2/3
  ; Debounce Fire
  lda DEBOUNCE            ; [40] + 3
  and #%00100000          ; [43] + 2
  bne WaitGameEnd         ; [45] + 2/3
  ; End Game
EndGame
  ; Fire Pressed
  lda #%00100000
  sta DEBOUNCE
  ; Reset Stack Pointer
  ldx #$FF
  txs
  ; Finish Game
  jmp FinishGame
ResetGameOverFire
  ; Clear Fire Button State
  lda DEBOUNCE            ; [41] + 3
  and #%11011111          ; [44] + 2
  sta DEBOUNCE            ; [46] + 3
WaitGameEnd
  jmp StartKernel         ; [49] + 3      = 52

GameOverFill
  ; Fill Well With Blocks
  ldy #FILL1              ; [36] + 2
  ldx #0                  ; [38] + 2
FillLoop
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  lda GRID+2,X            ; [7] + 4
  and #%01000000          ; [11] + 2      Test Fill Bit
  bne NextFill            ; [13] + 2/3
  lda #FILL0              ; [15] + 2
  sta GRID+0,X            ; [17] + 4
  sty GRID+1,X            ; [21] + 4
  ora #%01000000          ; [25] + 2      Set Fill Bit
  sta GRID+2,X            ; [27] + 4
  sty GRID+3,X            ; [31] + 4
  ldx #GAMEOVERTIME       ; [35] + 2
  stx TIMER               ; [37] + 3
  jmp StartKernel         ; [40] + 3      = 43
NextFill
  inx                     ; [16] + 2
  inx                     ; [18] + 2
  inx                     ; [20] + 2
  inx                     ; [22] + 2
  cpx #72                 ; [24] + 2
  bcc FillLoop            ; [26] + 2/3
  jmp StartKernel         ; [28] + 3      = 31

; -----------------------------------------------------------------------------

FlashMode
  ; Continue Flashing Until Times Is Zero
  lda TIMER               ; [21] + 3
  and #%00011111          ; [24] + 2
  ; Compress When Timer = 0
  bne DoFlash             ; [26] + 2/3
  jmp CompressMode        ; [28] + 3
DoFlash
  ; Add Score When Timer = 1
  cmp #1                  ; [29] + 2
  bne DoFlash2            ; [31] + 2/3
  jmp AddScore            ; [33] + 3
DoFlash2
  ; Calculate Flash
  lsr                     ; [34] + 2
  lsr                     ; [36] + 2
  lsr                     ; [38] + 2
  ldx #FLASHON            ; [40] + 2
  bcs FlashOff            ; [42] + 2/3
  bcc FlashStore          ; [44] + 3
FlashOff
  ldx #FLASHOFF           ; [45] + 2
FlashStore
  stx TEMP                ; [47] + 3
  ldx #68                 ; [50] + 2
FlashLoop
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ; Check For Complete Line
  lda GRID+0,X            ; [7] + 4
  and #%01000000          ; [11] + 2
  beq NextFlashA          ; [13] + 2/3
  ; Update Line
  lda GRID+2,X            ; [15] + 4
  and #%10111111          ; [19] + 2
  ora TEMP                ; [21] + 3
  sta GRID+2,X            ; [24] + 4
NextFlashA
  dex                     ; [28] + 2
  dex                     ; [30] + 2
  dex                     ; [32] + 2
  dex                     ; [34] + 2
  ; Check For Complete Line
  lda GRID+0,X            ; [36] + 4
  and #%01000000          ; [40] + 2
  beq NextFlashB          ; [42] + 2/3
  ; Update Line
  lda GRID+2,X            ; [44] + 4
  and #%10111111          ; [48] + 2
  ora TEMP                ; [50] + 3
  sta GRID+2,X            ; [53] + 4
NextFlashB
  dex                     ; [57] + 2
  dex                     ; [59] + 2
  dex                     ; [61] + 2
  dex                     ; [63] + 2
  bpl FlashLoop           ; [65] + 2/3
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  jmp StartKernel         ; [7] + 3       = 10
  
; -----------------------------------------------------------------------------

CompressMode
  ; Compress Grid
  ldx #0                  ; [31] + 2      0
  bit GRID+0              ; [33] + 3
  bvs CompressLoop        ; [36] + 2/3
  ldx #4                  ; [38] + 2      1
  bit GRID+4              ; [40] + 3
  bvs CompressLoop        ; [43] + 2/3
  ldx #8                  ; [45] + 2      2
  bit GRID+8              ; [47] + 3
  bvs CompressLoop        ; [50] + 2/3
  ldx #12                 ; [52] + 2      3
  bit GRID+12             ; [54] + 3
  bvs CompressLoop        ; [57] + 2/3
  ldx #16                 ; [59] + 2      4
  bit GRID+16             ; [61] + 3
  bvs CompressLoop        ; [64] + 2/3
  ldx #20                 ; [66] + 2       5
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  bit GRID+20             ; [7] + 3
  bvs CompressLoop        ; [10] + 2/3
  ldx #24                 ; [12] + 2      6
  bit GRID+24             ; [14] + 3
  bvs CompressLoop        ; [17] + 2/3
  ldx #28                 ; [19] + 2      7
  bit GRID+28             ; [21] + 3
  bvs CompressLoop        ; [24] + 2/3
  ldx #32                 ; [26] + 2      8
  bit GRID+32             ; [28] + 3
  bvs CompressLoop        ; [31] + 2/3
  ldx #36                 ; [33] + 2      9
  bit GRID+36             ; [35] + 3
  bvs CompressLoop        ; [38] + 2/3
  ldx #40                 ; [40] + 2      10
  bit GRID+40             ; [42] + 3
  bvs CompressLoop        ; [45] + 2/3
  ldx #44                 ; [47] + 2      11
  bit GRID+44             ; [49] + 3
  bvs CompressLoop        ; [52] + 2/3
  ldx #48                 ; [54] + 2      12
  bit GRID+48             ; [56] + 3
  bvs CompressLoop        ; [59] + 2/3
  ldx #52                 ; [61] + 2      13
  bit GRID+52             ; [63] + 3
  bvs CompressLoop        ; [66] + 2/3
  ldx #56                 ; [68] + 2      14
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  bit GRID+56             ; [7] + 3
  bvs CompressLoop        ; [10] + 2/3
  ldx #60                 ; [12] + 2      15
  bit GRID+60             ; [14] + 3
  bvs CompressLoop        ; [17] + 2/3
  ldx #64                 ; [19] + 2      16
  bit GRID+64             ; [21] + 3
  bvs CompressLoop        ; [24] + 2/3
  bit GRID+68             ; [26] + 3      17
  bvs ClearTopLine        ; [29] + 2/3

  ; Finished Compression
  jmp NextShape           ; [31] + 3
  
CompressLoop
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  lda GRID+4,X            ; [7] + 4
  sta GRID+0,X            ; [11] + 4
  lda GRID+5,X            ; [15] + 4
  sta GRID+1,X            ; [19] + 4
  lda GRID+6,X            ; [23] + 4
  sta GRID+2,X            ; [27] + 4
  lda GRID+7,X            ; [31] + 4
  sta GRID+3,X            ; [35] + 4
  inx                     ; [39] + 2
  inx                     ; [41] + 2
  inx                     ; [43] + 2
  inx                     ; [45] + 2
  cpx #68                 ; [47] + 2
  bcc CompressLoop        ; [49] + 2/3
ClearTopLine
  ; Clear Top Line
  ldx #0                  ; [51] + 2
  stx GRID+68             ; [53] + 3
  stx GRID+69             ; [56] + 3
  stx GRID+70             ; [59] + 3
  stx GRID+71             ; [62] + 3
  
  ; Line Compressed
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ; Store Mode/Game Bits
  lax LINES+1             ; [7] + 3
  and #%11110000          ; [10] + 2
  sta TEMP                ; [12] + 3
  ; Increment Lines
  sed                     ; [15] + 2
  clc                     ; [17] + 2
  lda LINES+0             ; [19] + 3
  adc #1                  ; [22] + 2
  sta LINES+0             ; [24] + 3
  txa                     ; [27] + 2
  and #%00001111          ; [29] + 2
  adc #0                  ; [31] + 2
  and #%00001111          ; [33] + 2
  ora TEMP                ; [35] + 3
  sta LINES+1             ; [38] + 3
  cld                     ; [41] + 2
  ; Check for Level Up
  lda LINES+0             ; [43] + 3
  and #%00001111          ; [46] + 2
  bne NoNewLevel          ; [48] + 2/3
  ; New Level
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ; Store Height
  lax LEVEL               ; [7] + 3
  and #%11100000          ; [10] + 2
  sta TEMP                ; [12] + 3
  ; Check for Final Level (19)
  txa                     ; [15] + 2
  and #%00011111          ; [17] + 2
  cmp #19                 ; [19] + 2
  bcs EndCompress         ; [21] + 2/3
  ; Increment Level
  ; clc
  adc #1                  ; [23] + 2
  ora TEMP                ; [25] + 3
  sta LEVEL               ; [28] + 3
  jmp EndCompress         ; [31] + 3
NoNewLevel
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
EndCompress
  ; Compress Sound
  lda SFX                 ; [34] + 3
  beq PlayCompressSound   ; [37] + 2/3
  and #%11100000          ; [39] + 2
  cmp #5<<5               ; [41] + 2
  bcc EndCompressSound    ; [43] + 2/3
PlayCompressSound
  ldx #((5<<5)|3)         ; [45] + 2
  stx SFX                 ; [47] + 3
EndCompressSound
  jmp StartKernel         ; [50] + 3      = 53
  
NextShape
  ; Clear Modes
  lda LINES+1             ; [34] + 3
  and #%00111111          ; [37] + 2
  sta LINES+1             ; [39] + 3
  jmp Spawn               ; [42] + 3

; -----------------------------------------------------------------------------

SettleMode
  ; Check Timer (0 = New Shape)
  lda TIMER               ; [24] + 3
  and #%00011111          ; [27] + 2
  beq NextShape           ; [29] + 2/3
  ; Reset Line Count
  lda #0                  ; [31] + 2
  sta TEMP                ; [33] + 3
  ; Test For Completed Lines
  jmp CheckShapeLine      ; [36] + 3
EndCheckShapeLine
  ; Check Line Counter (in TEMP)
  lda TEMP                ; [10] + 3
  beq EndNewLines         ; [13] + 2/3
  ; Play Sound Effect
  asl                     ; [15] + 2
  asl                     ; [17] + 2
  asl                     ; [19] + 2
  asl                     ; [21] + 2
  asl                     ; [23] + 2
  ora #30                 ; [25] + 2      Sound Length
  sta SFX                 ; [27] + 3
  ; Store Lines in XPOS
  lda XPOS                ; [30] + 3
  and #%11110000          ; [33] + 2
  ora TEMP                ; [35] + 3
  sta XPOS                ; [38] + 3
  ; Set Flash Timer
  lda TIMER               ; [41] + 3
  ora #FLASHTIME          ; [44] + 2
  sta TIMER               ; [46] + 3
  ; Set Flash Mode
  lda LINES+1             ; [49] + 3
  and #%00111111          ; [52] + 2
  ora #%10000000          ; [54] + 2
  sta LINES+1             ; [56] + 3
EndNewLines
  jmp StartKernel         ; [59] + 3      = 62

; -----------------------------------------------------------------------------

Spawn
  ; Generate New Shape
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ; Defauly Row & Rotation
  ldy #18                 ; [7] + 2
  ; Get Drop Bit
  lda XPOS                ; [9] + 3
  and #%10000000          ; [12] + 2
  sta TEMP                ; [14] + 3
  ; Copy Previous Piece
  lda TIMER               ; [17] + 3
  and #%11100000          ; [20] + 2
  ; Increment Y For I Piece
  cmp #(1<<5)             ; [22] + 2
  beq DoYAdjust           ; [24] + 2/3
  bne NoYAdjust           ; [26] + 3
DoYAdjust
  iny                     ; [27] + 2      Increment Row
NoYAdjust
  lsr                     ; [29] + 2
  ora #4                  ; [31] + 2      Middle Position
  ora TEMP                ; [33] + 3
  sta XPOS                ; [36] + 3
  sty YPOS                ; [39] + 3
  ; Generate New Piece
  ldy RANDOM              ; [42] + 3
  lda Mod7,Y              ; [45] + 4
  sta TIMER               ; [49] + 3
  ; Set Drop & L/R Delays
  lda DEBOUNCE            ; [52] + 3
  ora #(VDELAY | HDELAY)  ; [55] + 2
  sta DEBOUNCE            ; [57] + 3
  
  ; Get Shape Details
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  LOADTEMPS               ; [7] + 51
  ; Check Overlap
  jsr CheckOverlap        ; [0] + 15
  bne FullGameOver        ; [15] + 2/3
  ; Reset Timer
  SETTIMER                ; [17] + 32
  ; Display Shape
  jmp ShapeCalc           ; [49] + 3      = 52

FullGameOver
  ; Set GameOver Mode
  lda LINES+1             ; [18] + 3
  ora #%11000000          ; [21] + 2
  sta LINES+1             ; [23] + 3
  ; Set Timer/Clear Next Piece
  ldx #GAMEOVERTIME       ; [26] + 2
  stx TIMER               ; [28] + 3
  ; Play Fail Sound
  ldx #((7<<5)|31)        ; [31] + 2
  stx SFX                 ; [33] + 3

ShapeCalc
  ; Recalculate Shape Values
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  SHAPEVALS               ; [7] + 33
  jmp ShowShape           ; [40] + 3      = 43

; -----------------------------------------------------------------------------

NormalMode
  ; Check Down Debounce
  lax DEBOUNCE            ; [23] + 3
  and #%11100111          ; [26] + 2
  sta TEMP                ; [28] + 3
  txa                     ; [31] + 2
  and #%00011000          ; [33] + 2
  beq JoyDown             ; [35] + 2/3
  sec                     ; [37] + 2
  sbc #8                  ; [39] + 2
  and #%00011000          ; [41] + 2
  ora TEMP                ; [43] + 3
  sta DEBOUNCE            ; [46] + 3
  jmp EndJoyDown          ; [49] + 3      = 52
JoyDown
  ; Check Down
  lda #%00100000          ; [38] + 2
  bit SWCHA               ; [40] + 3
  bne EndJoyDown          ; [43] + 2/3    = 56
  ; Debounce Down
  txa                     ; [45] + 2
  ora #VDELAY             ; [47] + 2
  sta DEBOUNCE            ; [49] + 3
  ; Drop Shape
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  jmp DropShape           ; [7] + 3       = 10
EndJoyDown

; -----------------------------------------------------------------------------

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Fetch Previous Joystick State (Left/Right only)
  lax DEBOUNCE            ; [7] + 3
  and #%11000000          ; [10] + 2
  sta TEMP                ; [12] + 3
  ; Fetch Current Joystick State (Left/Right only)
  lda SWCHA               ; [15] + 3
  and #%11000000          ; [18] + 2
  ; Reset Debounce If Joystick Has Moved
  cmp TEMP                ; [20] + 3
  bne ResetDebounce       ; [23] + 2/3
  ; Fetch Joystick & Button State
  txa                     ; [25] + 2
  and #%11111000          ; [27] + 2
  sta TEMP                ; [29] + 3
  ; Repeat Joystick Movement When Counter reaches Zero (Auto-Repeat)
  txa                     ; [32] + 2
  and #%00000111          ; [34] + 2
  beq JoyLeftRight        ; [36] + 2/3
  ; Decrement Debounce Counter
  sec                     ; [38] + 2
  sbc #1                  ; [40] + 2
  ora TEMP                ; [42] + 3
  sta DEBOUNCE            ; [45] + 3
  jmp EndJoyLeftRight     ; [48] + 3
ResetDebounce
  ; Update Joystick State & Reset Counter
  sta TEMP                ; [26] + 3
  txa                     ; [29] + 2
  and #%00111000          ; [31] + 2
  ora TEMP                ; [33] + 3
  sta DEBOUNCE            ; [36] + 3
JoyLeftRight
  ; Check for Left/Right
  bit DEBOUNCE            ; [39] + 3
  bpl JoyRight            ; [42] + 2/3
  bvc JoyLeft             ; [44] + 2/3
  jmp EndJoyLeftRight     ; [46] + 3
JoyLeft
  ; Move Left
  dec TEMPX               ; [47] + 5
  jmp TestPosition        ; [52] + 3
JoyRight
  ; Move Right
  inc TEMPX               ; [45] + 5
TestPosition
  ; Check For Overlap
  jsr CheckOverlap        ; [0] + 15
  beq StoreLeftRight      ; [15] + 2/3
  ; Restore Previous Value
  lda TEMPX2              ; [17] + 3
  sta TEMPX               ; [20] + 3
  jmp EndJoyLeftRight     ; [23] + 3
StoreLeftRight
  ; Store Position
  ldx TEMPX               ; [18] + 3
  inx                     ; [21] + 2
  bmi EndJoyLeftRight     ; [23] + 2/3    (Paranoia)
  stx TEMP                ; [25] + 3
  lda XPOS                ; [28] + 3
  and #%11110000          ; [31] + 2
  ora TEMP                ; [33] + 3
  sta XPOS                ; [36] + 3
  ; Debounce Left/Right
  lda DEBOUNCE            ; [39] + 3
  and #%11111000          ; [42] + 2
  ora #HDELAY             ; [44] + 2
  sta DEBOUNCE            ; [46] + 3
EndJoyLeftRight

; -----------------------------------------------------------------------------

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Check Fire Button
  lda INPT4               ; [7] + 3
  bmi ResetFire           ; [10] + 2/3
  ; Check Previous Button State
  lda DEBOUNCE            ; [12] + 3
  and #%00100000          ; [15] + 2
  bne EndJoyFire          ; [17] + 2/3    = 20
  ; Rotate Piece
  clc                     ; [19] + 2
  lda TEMPROT             ; [21] + 3
  adc #32                 ; [24] + 2
  and #%01100000          ; [26] + 2
  sta TEMPROT             ; [28] + 3
  ; Recalculate Shape Pointer
  lsr                     ; [31] + 2
  lsr                     ; [33] + 2
  lsr                     ; [35] + 2
  sta TEMP                ; [37] + 3
  ; Get Shape Pointer (Type * 32 + Rotation * 4)
  lda TEMPVAL             ; [40] + 3
  asl                     ; [43] + 2
  ora TEMP                ; [45] + 3
  sta SPTR                ; [48] + 3
  ; Check Overlap
  jsr CheckOverlap        ; [0] + 15
  beq StoreRotate         ; [15] + 2/3
  ; Restore Previous Values
  lda TEMPROT2            ; [17] + 3
  sta TEMPROT             ; [20] + 3
  lda SPTR2               ; [23] + 3
  sta SPTR                ; [26] + 3
  jmp EndJoyFire          ; [29] + 3      = 32
StoreRotate
  ; Set Fire Button State
  lda DEBOUNCE            ; [18] + 3
  ora #%00100000          ; [21] + 2
  sta DEBOUNCE            ; [23] + 3
  ; Update Rotation
  lda YPOS                ; [26] + 3
  and #%10011111          ; [29] + 2
  ora TEMPROT             ; [31] + 3
  sta YPOS                ; [34] + 3
  ; Rotate Sound
  lda SFX                 ; [37] + 3
  beq PlayRotSound        ; [40] + 2/3
  and #%11100000          ; [42] + 2
  cmp #6<<5               ; [44] + 2
  bcc EndJoyFire          ; [46] + 2/3
PlayRotSound
  ldx #((6<<5)|7)         ; [48] + 2
  stx SFX                 ; [50] + 3
  jmp EndJoyFire          ; [53] + 3      = 56
ResetFire
  ; Clear Fire Button State
  lda DEBOUNCE            ; [13] + 3
  and #%11011111          ; [16] + 2
  sta DEBOUNCE            ; [18] + 3
EndJoyFire

; -----------------------------------------------------------------------------

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Check Drop Timer
  lda TIMER               ; [7] + 3
  and #%00011111          ; [10] + 2
  bne EndDropShape        ; [12] + 2/3    = 15
DropShape
  ; New Y Position
  dec TEMPY               ; [14] + 5
  bmi StartSettle         ; [19] + 2/3
  ; Check Overlap
  jsr CheckOverlap        ; [0] + 15
  bne StartSettle2        ; [15] + 2/3
  ; Update Position
  lda YPOS                ; [17] + 3
  and #%11100000          ; [20] + 2
  ora TEMPY               ; [22] + 3
  sta YPOS                ; [25] + 3
  ; Reset Timer
  SETTIMER                ; [28] + 32
  jmp ShowShape           ; [60] + 3      = 63
StartSettle
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
StartSettle2
  ; Restore Previous Value
  inc TEMPY               ; [18] + 5
  ; Settle Mode
  lda LINES+1             ; [23] + 3
  ora #%01000000          ; [26] + 2
  sta LINES+1             ; [28] + 3
  ; Settle Timer (2 phases)
  lda TIMER               ; [31] + 3
  and #%11100000          ; [34] + 2
  ora #%00000010          ; [36] + 2
  sta TIMER               ; [38] + 3
  ; Settle Sound
  lda SFX                 ; [41] + 3
  beq PlaySettleSound     ; [44] + 2/3
  and #%11100000          ; [46] + 2
  cmp #5<<5               ; [48] + 2
  bcc EndDropShape        ; [50] + 2/3    = 51
PlaySettleSound
  ldx #((5<<5)|3)         ; [52] + 2
  stx SFX                 ; [54] + 3
EndDropShape

; -----------------------------------------------------------------------------

ShowShape
  ; Draw Shape
  jsr DrawShape           ; [0] + 13
  jmp StartKernel         ; [13] + 3      = 16

; -----------------------------------------------------------------------------
; OVERSCAN REGION (AFTER KERNEL)
; -----------------------------------------------------------------------------

OverLogic
  ; Check for Game Over
  ldx LINES+1             ; [16] + 3
  cpx #%11000000          ; [19] + 2
  bcc UpdateTimer         ; [21] + 2/3
  
  ; Slow Timer During GameOver
  lsr                     ; [23] + 2
  bcs SkipDelete          ; [25] + 2/3    = 28
  lsr                     ; [27] + 2
  bcs SkipDelete          ; [29] + 2/3    = 32

UpdateTimer
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Update Drop Timer
  lax TIMER               ; [7] + 3
  and #%11100000          ; [10] + 2
  sta TEMP                ; [12] + 3
  txa                     ; [15] + 2
  and #%00011111          ; [17] + 2
  beq SkipTimer           ; [19] + 2/3
  sec                     ; [21] + 2
  sbc #1                  ; [23] + 2
  ora TEMP                ; [25] + 3
  sta TIMER               ; [28] + 3
SkipTimer

  ; Check for Game Over
  lax LINES+1             ; [31] + 3
  cmp #%11000000          ; [34] + 2
  bcc DeleteShape         ; [36] + 2/3

SkipDelete
  jmp EndOverLogic        ; [38] + 3

; -----------------------------------------------------------------------------

DeleteShape
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Delete Shape If No Mode Set
  lda LINES+1             ; [7] + 3
  and #%11000000          ; [10] + 2
  bne EndOverLogic        ; [12] + 2/3

  ; Set Zero Values To Delete
  ; lda #0
  sta VALUE1              ; [14] + 3
  sta VALUE2              ; [17] + 3
  sta VALUE3              ; [20] + 3
  sta VALUE4              ; [23] + 3
  
  ; Get Shape Details
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  LOADTEMPS               ; [7] + 51

  ; Delete Shape
  jsr DrawShape           ; [0] + 13

EndOverLogic
  WAIT_OVERSCAN_MUSIC
  jmp MainLoop

  ; Start Positions (*4)
StartPos
  DC.B  0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68

  DC.B  "X0"

; -----------------------------------------------------------------------------
; SUBROUTINES
; -----------------------------------------------------------------------------

  ALIGN 256

  ; Add Score
  ; XPOS = Lines
AddScore
  ; Get Level
  lda LEVEL               ; [40] + 3
  and #%00011111          ; [43] + 2
  tax                     ; [45] + 2
  lda XPOS                ; [47] + 3
  and #%00001111          ; [50] + 2
  tay                     ; [52] + 2
  sed                     ; [54] + 2
  sta WSYNC               ; [0]
AddScoreLoop
  ; Add Score (Part 1)
  PLAY_MUSIC              ; [0] + 7
  clc                     ; [7] + 2
  lda SCORE+0             ; [9] + 3
  adc ScoreLo,Y           ; [12] + 4
  sta SCORE+0             ; [16] + 3
  lda SCORE+1             ; [19] + 3
  adc ScoreHi,Y           ; [22] + 4
  sta SCORE+1             ; [26] + 3
  lda SCORE+2             ; [29] + 3
  adc #0                  ; [32] + 2
  sta SCORE+2             ; [34] + 3
  dex                     ; [37] + 2
  bmi EndAddScoreLoop     ; [39] + 2/3
  ; Add Score (Part 2)
  clc                     ; [41] + 2
  lda SCORE+0             ; [43] + 3
  adc ScoreLo,Y           ; [46] + 4
  sta SCORE+0             ; [50] + 3
  lda SCORE+1             ; [53] + 3
  adc ScoreHi,Y           ; [56] + 4
  sta SCORE+1             ; [60] + 3
  lda SCORE+2             ; [63] + 3
  adc #0                  ; [66] + 2
  sta SCORE+2             ; [68] + 3
  dex                     ; [71] + 2
  bpl AddScoreLoop        ; [73] + 2/3
EndAddScore
  PLAY_MUSIC              ; [75] + 7
EndAddScoreLoop
  cld                     ; [42] + 2
  jmp StartKernel         ; [44] + 3      = 47

; -----------------------------------------------------------------------------

CheckShapeLine
  ; Shape Part 1
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sec                     ; [7] + 2
  ldx SPTR                ; [9] + 3
  lda TEMPY               ; [12] + 3
  sbc ShapeData+19,X      ; [15] + 4
  jsr CheckRow            ; [19] + 6
  ; Shape Part 2
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sec                     ; [7] + 2
  ldx SPTR                ; [9] + 3
  lda TEMPY               ; [12] + 3
  sbc ShapeData+18,X      ; [15] + 4
  jsr CheckRow            ; [19] + 6
  ; Shape Part 3
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sec                     ; [7] + 2
  ldx SPTR                ; [9] + 3
  lda TEMPY               ; [12] + 3
  sbc ShapeData+17,X      ; [15] + 4
  jsr CheckRow            ; [19] + 6
  ; Shape Part 4
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sec                     ; [7] + 2
  ldx SPTR                ; [9] + 3
  lda TEMPY               ; [12] + 3
  sbc ShapeData+16,X      ; [15] + 4
  jsr CheckRow            ; [19] + 6
  ; Check For New Lines (Count in TEMP)
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  jmp EndCheckShapeLine   ; [7] + 3     = 10

; -----------------------------------------------------------------------------

  ; Check for Complete Row
  ; A = Row (0-18)
CheckRow
  ; Check For Invalid Row
  cmp #18                 ; [25] + 2
  bcs EndCheckLine        ; [27] + 2/3
  asl                     ; [29] + 2
  asl                     ; [31] + 2
  tay                     ; [33] + 2
  lax GRID+0,Y            ; [35] + 4
  and #%00000111          ; [39] + 2
  beq EndCheckLine        ; [41] + 2/3
  txa                     ; [43] + 2
  rol                     ; [45] + 2
  and #%01110000          ; [47] + 2
  beq EndCheckLine        ; [49] + 2/3
  lax GRID+1,Y            ; [51] + 4
  and #%00000111          ; [55] + 2
  beq EndCheckLine        ; [57] + 2/3
  txa                     ; [59] + 2
  and #%00111000          ; [61] + 2
  beq EndCheckLine        ; [63] + 2/3
  txa                     ; [65] + 2
  ror                     ; [67] + 2
  and #%11100000          ; [69] + 2
  beq EndCheckLine        ; [71] + 2/3

  PLAY_MUSIC              ; [73] + 7
  
  lax GRID+2,Y            ; [4] + 4
  and #%00000111          ; [8] + 2
  beq EndCheckLine        ; [10] + 2/3
  txa                     ; [12] + 2
  rol                     ; [14] + 2
  and #%01110000          ; [16] + 2
  beq EndCheckLine        ; [18] + 2/3
  lax GRID+3,Y            ; [20] + 4
  and #%00000111          ; [24] + 2
  beq EndCheckLine        ; [26] + 2/3
  txa                     ; [28] + 2
  and #%00111000          ; [30] + 2
  beq EndCheckLine        ; [32] + 2/3
  txa                     ; [34] + 2
  ror                     ; [36] + 2
  and #%11100000          ; [38] + 2
  beq EndCheckLine        ; [40] + 2/3
FoundLine
  ; Check If Line Already Marked
  lax GRID+0,Y            ; [42] + 4
  and #%01000000          ; [46] + 2
  bne EndCheckLine        ; [48] + 2/3
  ; Mark Line
  txa                     ; [50] + 2
  ora #%01000000          ; [52] + 2
  sta GRID+0,Y            ; [54] + 4
  ; Increment Counter
  inc TEMP                ; [58] + 5
EndCheckLine
  rts                     ; [63] + 6 = 69

  ; Shape Colours for Preview
PreviewCols
  DC.B  BCOL0, BCOL1, BCOL2, BCOL3, BCOL4, BCOL5, BCOL6, BCOL7
EndPreviewCols
  if (>PreviewCols != >EndPreviewCols)
    echo "WARNING: PreviewCols crosses a page boundary!"
  endif

  DC.B  "X1"

; -----------------------------------------------------------------------------

  ALIGN 256
  
  ; Next Random Number
GetRandom
  lda RANDOM              ; [0] + 3
  beq ResetRandom         ; [3] + 2/3
  bne NoResetRandom       ; [5] + 3
ResetRandom
  lda #RANDOMSEED         ; [6] + 2
NoResetRandom
  asl                     ; [8] + 2
  bcs RndEor              ; [10] + 2/3
  bcc NoRndEor            ; [12] + 3
RndEor
  eor #$AF                ; [13] + 2
NoRndEor
  sta RANDOM              ; [15] + 3
  rts                     ; [18] + 6    = 24

; -----------------------------------------------------------------------------

; Check Shape Overlap
; TEMPX = Column (0-9)
; TEMPY = Row (0-19)
; SPTR = Shape Pointer
CheckOverlap
  ; Part 1
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldy SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+19,Y      ; [15] + 4
  tax                     ; [19] + 2
  clc                     ; [21] + 2
  lda ShapeData+3,Y       ; [23] + 4
  adc TEMPX               ; [27] + 3
  GETDATA EndOverlap1     ; [30] + 39/40
EndOverlap1
  bne FoundOverlap1       ; [70] + 2/3

  ; Part 2
CheckOverlap2
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldy SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+18,Y      ; [15] + 4
  tax                     ; [19] + 2
  clc                     ; [21] + 2
  lda ShapeData+2,Y       ; [23] + 4
  adc TEMPX               ; [27] + 3
  GETDATA EndOverlap2     ; [30] + 39
  beq CheckOverlap3       ; [69] + 2/4*
FoundOverlap1
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  lda #1                  ; [7] + 2
  rts                     ; [9] + 6     = 15

; Shape Preview Data (Part 0)
Previews0
  DC.B  %00000000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %01110000
  DC.B  %00100000
  DC.B  %00110000
  DC.B  %01110000
  DC.B  %01100000
EndPreviews0
  if (>Previews0 != >EndPreviews0)
    echo "WARNING: Previews0 crosses a page boundary!"
  endif

  DC.B  "X2"
  ALIGN 256

  ; Part 3
EndOverlap2
  bne FoundOverlap2       ; [70] + 2/3
CheckOverlap3
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldy SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+17,Y      ; [15] + 4
  tax                     ; [19] + 2
  clc                     ; [21] + 2
  lda ShapeData+1,Y       ; [23] + 4
  adc TEMPX               ; [27] + 3
  GETDATA EndOverlap3     ; [30] + 39/40
EndOverlap3
  beq CheckOverlap4       ; [70] + 2/3
FoundOverlap2
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  lda #1                  ; [7] + 2
  rts                     ; [9] + 6     = 15

  ; Part 4
CheckOverlap4
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldy SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+16,Y      ; [15] + 4
  tax                     ; [19] + 2
  clc                     ; [21] + 2
  lda ShapeData+0,Y       ; [23] + 4
  adc TEMPX               ; [27] + 3
  GETDATA EndOverlap4     ; [30] + 39/40
EndOverlap4
  bne FoundOverlap2       ; [70] + 2/3
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  lda #0                  ; [7] + 2
  rts                     ; [9] + 6    = 15

; Shape Preview Data (Part 1)
Previews1
  DC.B  %00000000
  DC.B  %00000000
  DC.B  %00010000
  DC.B  %01000000
  DC.B  %00100000
  DC.B  %01100000
  DC.B  %00100000
  DC.B  %00110000
EndPreviews1
  if (>Previews1 != >EndPreviews1)
    echo "WARNING: Previews1 crosses a page boundary!"
  endif

  DC.B  "X3"

; -----------------------------------------------------------------------------

  ALIGN 256

  ; Draw Current Shape on Grid
  ; TEMPX = Column (0-9)
  ; TEMPY = Row (0-19)
  ; SPTR = Shape Pointer
  ; VALUE1-4 = Shape Colour (0-7)
DrawShape
  ; Part 1
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldx SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+19,X      ; [15] + 4
  tay                     ; [19] + 2
  lda ShapeData+3,X       ; [21] + 4
  ldx ShapePosition,Y     ; [25] + 4
  bmi DrawShape2A         ; [29] + 2/3
  clc                     ; [31] + 2
  adc TEMPX               ; [33] + 3
  SETDATA DrawShape2B, DrawShape2A    ; [36] + 37/40

  ; Part 2
DrawShape2A
  sta WSYNC               ; [0]
DrawShape2B
  PLAY_MUSIC              ; [0] + 7
  ldx SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+18,X      ; [15] + 4
  tay                     ; [19] + 2
  lda ShapeData+2,X       ; [21] + 4
  ldx ShapePosition,Y     ; [25] + 4
  bmi DrawShape2C         ; [29] + 2/3
  clc                     ; [31] + 2
  adc TEMPX               ; [33] + 3
  SETDATA DrawShape3B, DrawShape2C    ; [36] + 37/40
  jmp DrawShape3B         ; [73] + 3
DrawShape2C
  jmp DrawShape3A

   ; Binary To BCD Table (21 Entries)
LevelBCDTable
  DC.B  %00000000, %00000001, %00000010, %00000011, %00000100
  DC.B  %00000101, %00000110, %00000111, %00001000, %00001001
  DC.B  %00010000, %00010001, %00010010, %00010011, %00010100
  DC.B  %00010101, %00010110, %00010111, %00011000, %00011001
  DC.B  %00100000
EndLevelBCDTable
  if (>LevelBCDTable != >EndLevelBCDTable)
    echo "WARNING: LevelBCDTable crosses a page boundary!"
  endif

  ; Digit Offsets for Score Display
ScoreDigits
  DC.B  <L_00-<LChars, <L_11-<LChars, <L_22-<LChars, <L_33-<LChars
  DC.B  <L_44-<LChars, <L_55-<LChars, <L_66-<LChars, <L_77-<LChars
  DC.B  <L_88-<LChars, <L_99-<LChars
EndScoreDigits
  if (>ScoreDigits != >EndScoreDigits)
    echo "WARNING: ScoreDigits pointers cross a page boundary!"
  endif

  DC.B  "X4"
  ALIGN 256

  ; Part 3
DrawShape3A
  sta WSYNC               ; [0]
DrawShape3B
  PLAY_MUSIC              ; [0] + 7
  ldx SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+17,X      ; [15] + 4
  tay                     ; [19] + 2
  lda ShapeData+1,X       ; [21] + 4
  ldx ShapePosition,Y     ; [25] + 4
  bmi DrawShape4A         ; [29] + 2/3
  clc                     ; [31] + 2
  adc TEMPX               ; [33] + 3
  SETDATA DrawShape4B, DrawShape4A   ; [36] + 37/40

  ; Part 4
DrawShape4A
  sta WSYNC               ; [0]
DrawShape4B
  PLAY_MUSIC              ; [0] + 7
  ldx SPTR                ; [7] + 3
  sec                     ; [10] + 2
  lda TEMPY               ; [12] + 3
  sbc ShapeData+16,X      ; [15] + 4
  tay                     ; [19] + 2
  lda ShapeData+0,X       ; [21] + 4
  ldx ShapePosition,Y     ; [25] + 4
  bmi DrawShape5A         ; [29] + 2/3
  clc                     ; [31] + 2
  adc TEMPX               ; [33] + 3
  SETDATA DrawShape5B, DrawShape5A   ; [36] + 37/40

DrawShape5A
  sta WSYNC               ; [0]
DrawShape5B
  PLAY_MUSIC              ; [0] + 7
  rts                     ; [7] + 6     = 13

; -----------------------------------------------------------------------------
; GAME DATA
; -----------------------------------------------------------------------------

  ; Level Speeds (Levels 1 -> 20)
Levels
  IF (PALMODE)
  DC.B  26, 24, 22, 20, 18
  DC.B  16, 15, 14, 13, 12
  DC.B  11, 10, 9, 8, 7
  DC.B  6, 5, 4, 3, 2
  ELSE
  DC.B  31, 29, 27, 25, 23
  DC.B  21, 19, 17, 15, 13
  DC.B  12, 11, 10, 9, 8
  DC.B  7, 6, 5, 4, 3
  ENDIF
EndLevels
  if (>Levels != >EndLevels)
    echo "WARNING: Levels Data Crosses Page Boundary!"
  endif

  ; Scoring Table
ScoreLo
  DC.B  %00000000         ; 0     (0 Lines)
  DC.B  %01000000         ; 40    (1 Line)
  DC.B  %00000000         ; 0
ScoreHi
  DC.B  %00000000         ; 0
  DC.B  %00000000         ; 0
  DC.B  %00000001         ; 100   (2 Lines)
  DC.B  %00000011         ; 300   (3 Lines)
  DC.B  %00010010         ; 1200  (4 Lines = Chetiry!)

  DC.B  "X5"

  ALIGN 256

  ; Row Offsets for Shape Drawing (256 Bytes)
ShapePosition
  DC.B  0, 4, 8, 12, 16, 20, 24, 28, 32
  DC.B  36, 40, 44, 48, 52, 56, 60, 64, 68
  DS.B  238, $FF

  ALIGN 256

  ; Shape Probabilities (256 bytes)
  ; (((Pos % 7) + 1) * 32)
Mod7
  DC.B  0, 32, 64, 96, 128, 160, 192, 224
  DC.B  32, 64, 96, 128, 160, 192, 224, 32
  DC.B  64, 96, 128, 160, 192, 224, 32, 64
  DC.B  96, 128, 160, 192, 224, 32, 64, 96
  DC.B  128, 160, 192, 224, 32, 64, 96, 128
  DC.B  160, 192, 224, 32, 64, 96, 128, 160
  DC.B  192, 224, 32, 64, 96, 128, 160, 192
  DC.B  224, 32, 64, 96, 128, 160, 192, 224
  DC.B  32, 64, 96, 128, 160, 192, 224, 32
  DC.B  64, 96, 128, 160, 192, 224, 32, 64
  DC.B  96, 128, 160, 192, 224, 32, 64, 96
  DC.B  128, 160, 192, 224, 32, 64, 96, 128
  DC.B  160, 192, 224, 32, 64, 96, 128, 160
  DC.B  192, 224, 32, 64, 96, 128, 160, 192
  DC.B  224, 32, 64, 96, 128, 160, 192, 224
  DC.B  32, 64, 96, 128, 160, 192, 224, 32
  DC.B  64, 96, 128, 160, 192, 224, 32, 64
  DC.B  96, 128, 160, 192, 224, 32, 64, 96
  DC.B  128, 160, 192, 224, 32, 64, 96, 128
  DC.B  160, 192, 224, 32, 64, 96, 128, 160
  DC.B  192, 224, 32, 64, 96, 128, 160, 192
  DC.B  224, 32, 64, 96, 128, 160, 192, 224
  DC.B  32, 64, 96, 128, 160, 192, 224, 32
  DC.B  64, 96, 128, 160, 192, 224, 32, 64
  DC.B  96, 128, 160, 192, 224, 32, 64, 96
  DC.B  128, 160, 192, 224, 32, 64, 96, 128
  DC.B  160, 192, 224, 32, 64, 96, 128, 160
  DC.B  192, 224, 32, 64, 96, 128, 160, 192
  DC.B  224, 32, 64, 96, 128, 160, 192, 224
  DC.B  32, 64, 96, 128, 160, 192, 224, 32
  DC.B  64, 96, 128, 160, 192, 224, 32, 64
  DC.B  96, 128, 160, 192, 224, 32, 64, 96

  ALIGN 256

  ; Shapes Data (256 bytes)
ShapeData
NoShape                   ; 0
  DC.B  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  DC.B  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
IShape                    ; 32
  DC.B  0, 1, 2, 3, 1, 1, 1, 1, 0, 1, 2, 3, 1, 1, 1, 1
  DC.B  2, 2, 2, 2, 0, 1, 2, 3, 2, 2, 2, 2, 0, 1, 2, 3
JShape                    ; 64
  DC.B  0, 1, 2, 2, 1, 1, 1, 0, 0, 0, 1, 2, 1, 1, 1, 2
  DC.B  1, 1, 1, 2, 0, 1, 2, 2, 0, 1, 1, 1, 0, 1, 2, 0
LShape                    ; 96
  DC.B  0, 0, 1, 2, 0, 1, 1, 1, 0, 1, 2, 2, 1, 1, 1, 2
  DC.B  1, 2, 1, 1, 0, 0, 1, 2, 1, 1, 1, 0, 0, 1, 2, 2
OShape                    ; 128
  DC.B  1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2
  DC.B  1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2, 1, 1, 2, 2
SShape                    ; 160
  DC.B  0, 1, 1, 2, 0, 0, 1, 1, 0, 1, 1, 2, 0, 0, 1, 1
  DC.B  2, 2, 1, 1, 0, 1, 1, 2, 2, 2, 1, 1, 0, 1, 1, 2
TShape                    ; 192
  DC.B  0, 1, 2, 1, 1, 1, 1, 0, 0, 1, 2, 1, 1, 1, 1, 2
  DC.B  1, 1, 1, 2, 0, 1, 2, 1, 1, 1, 1, 0, 0, 1, 2, 1
ZShape                    ; 224
  DC.B  0, 1, 1, 2, 1, 1, 0, 0, 0, 1, 1, 2, 1, 1, 0, 0
  DC.B  1, 1, 2, 2, 0, 1, 1, 2, 1, 1, 2, 2, 0, 1, 1, 2

; -----------------------------------------------------------------------------
; KERNEL SETUP
; -----------------------------------------------------------------------------

  ALIGN 256
  
; Load Score and Level Data For Kernel
StartKernel
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Shape Preview 
  lda TIMER               ; [7] + 3
  and #%11100000          ; [10] + 2
  lsr                     ; [12] + 2
  lsr                     ; [14] + 2
  lsr                     ; [16] + 2
  lsr                     ; [18] + 2
  lsr                     ; [20] + 2
  tax                     ; [22] + 2
  lda PreviewCols,X       ; [24] + 4
  sta COLUPF              ; [28] + 3
  lda Previews0,X         ; [31] + 4
  sta PFBUFF1             ; [35] + 3
  lda Previews1,X         ; [38] + 4
  sta PFBUFF0             ; [42] + 3
  lda #0                  ; [45] + 2
  sta CTRLPF              ; [47] + 3

  ; Set Stack To Buffer
  ldx #SBUFF+29           ; [50] + 2
  txs                     ; [52] + 2
  
  ; Load Level Number
  lda LEVEL               ; [54] + 3
  and #%00011111          ; [57] + 2
  tay                     ; [59] + 2
  lax LevelBCDTable,Y     ; [61] + 4
  lsr                     ; [65] + 2
  lsr                     ; [67] + 2
  lsr                     ; [69] + 2
  lsr                     ; [71] + 2
  tay                     ; [73] + 2
  PLAY_MUSIC              ; [75] + 7

  ; Store Level Number
  txa                     ; [6] + 2
  ldx ScoreDigits,Y       ; [8] + 4
  and #%00001111          ; [12] + 2
  tay                     ; [14] + 2
  lda ScoreDigits,Y       ; [16] + 4
  tay                     ; [20] + 2
  lda LChars+4,X          ; [22] + 4
  ora RChars+4,Y          ; [26] + 4
  pha                     ; [30] + 3
  
  ; Set Coarse Position
  sta RESP0               ; [33] + 3    = 36
  sta RESP1               ; [36] + 3    = 39
  
  lda LChars+3,X          ; [39] + 4
  ora RChars+3,Y          ; [43] + 4
  pha                     ; [47] + 3
  lda LChars+2,X          ; [50] + 4
  ora RChars+2,Y          ; [54] + 4
  pha                     ; [58] + 3
  lda LChars+1,X          ; [61] + 4
  ora RChars+1,Y          ; [65] + 4
  pha                     ; [69] + 3

  ; Reset Sprite Movements
  sta HMCLR               ; [72] + 3
  PLAY_MUSIC              ; [75] + 7

  lda LChars+0,X          ; [6] + 4
  ora RChars+0,Y          ; [10] + 4
  pha                     ; [14] + 3

  ; Load Lower Lines Digits
  lda LINES+0             ; [17] + 3
  lsr                     ; [20] + 2
  lsr                     ; [22] + 2
  lsr                     ; [24] + 2
  lsr                     ; [26] + 2
  tax                     ; [28] + 2
  ldy ScoreDigits,X       ; [30] + 4
  lda LINES+0             ; [34] + 3
  and #%00001111          ; [37] + 2
  tax                     ; [39] + 2
  lda ScoreDigits,X       ; [41] + 4
  tax                     ; [45] + 2
  lda LChars+4,Y          ; [47] + 4
  ora RChars+4,X          ; [51] + 4
  pha                     ; [55] + 3
  lda LChars+3,Y          ; [58] + 4
  ora RChars+3,X          ; [62] + 4
  pha                     ; [66] + 3

  ; Set Fine Position
  lda #%00010000          ; [69] + 2
  sta HMP1                ; [71] + 3
  PLAY_MUSIC              ; [74] + 7

  lda LChars+2,Y          ; [5] + 4
  ora RChars+2,X          ; [9] + 4
  pha                     ; [13] + 3
  lda LChars+1,Y          ; [16] + 4
  ora RChars+1,X          ; [20] + 4
  pha                     ; [24] + 3
  lda LChars+0,Y          ; [27] + 4
  ora RChars+0,X          ; [31] + 4
  pha                     ; [35] + 3

  ; Load Upper Lines Digits
  lda LINES+1             ; [38] + 3
  and #%00001111          ; [41] + 2
  tax                     ; [43] + 2
  ldy ScoreDigits,X       ; [45] + 4
  lda RChars+4,Y          ; [49] + 4
  pha                     ; [53] + 3
  lda RChars+3,Y          ; [56] + 4
  pha                     ; [60] + 3
  lda RChars+2,Y          ; [63] + 4
  pha                     ; [67] + 3
  lda RChars+1,Y          ; [70] + 4
  pha                     ; [74] + 3
  PLAY_MUSIC              ; [1] + 7
  lda RChars+0,Y          ; [8] + 4
  pha                     ; [12] + 3

  ; Load Lower Score
  lda SCORE+0             ; [15] + 3
  lsr                     ; [18] + 2
  lsr                     ; [20] + 2
  lsr                     ; [22] + 2
  lsr                     ; [24] + 2
  tax                     ; [26] + 2
  ldy ScoreDigits,X       ; [28] + 4
  lda SCORE+0             ; [32] + 3
  and #%00001111          ; [35] + 2
  tax                     ; [37] + 2
  lda ScoreDigits,X       ; [39] + 4
  tax                     ; [43] + 2
  lda LChars+4,Y          ; [45] + 4
  ora RChars+4,X          ; [49] + 4
  pha                     ; [53] + 3
  lda LChars+3,Y          ; [56] + 4
  ora RChars+3,X          ; [60] + 4
  pha                     ; [64] + 3
  
  ; Set Score Sprite Colours
  IF (PALMODE)
  lda #$3E                ; [67] + 2
  ELSE
  lda #$DE                ; [67] + 2
  ENDIF
  sta COLUP0              ; [69] + 3
  sta COLUP1              ; [72] + 3
  
  jmp ScoreKernel3        ; [75] + 3
EndScoreKernel2
  if (>StartKernel != >EndScoreKernel2)
    echo "WARNING: Score Kernel Setup Crosses Page Boundary!"
  endif

  DC.B  "X6"
  ALIGN 256

ScoreKernel3
  PLAY_MUSIC              ; [2] + 7
  
  lda LChars+2,Y          ; [9] + 4
  ora RChars+2,X          ; [13] + 4
  pha                     ; [17] + 3
  lda LChars+1,Y          ; [20] + 4
  ora RChars+1,X          ; [24] + 4
  pha                     ; [28] + 3
  lda LChars+0,Y          ; [31] + 4
  ora RChars+0,X          ; [35] + 4
  pha                     ; [39] + 3

  ; Load Middle Score Digits
  lda SCORE+1             ; [42] + 3
  lsr                     ; [45] + 2
  lsr                     ; [47] + 2
  lsr                     ; [49] + 2
  lsr                     ; [51] + 2
  tax                     ; [53] + 2
  ldy ScoreDigits,X       ; [55] + 4
  lda SCORE+1             ; [59] + 3
  and #%00001111          ; [62] + 2
  tax                     ; [64] + 2  
  lda ScoreDigits,X       ; [66] + 4
  tax                     ; [70] + 2
  
  ; 3 Copies Close
  lda #%00000011          ; [72] + 2
  sta NUSIZ0              ; [74] + 3
  PLAY_MUSIC              ; [1] + 7
  
  lda LChars+4,Y          ; [8] + 4
  ora RChars+4,X          ; [12] + 4
  pha                     ; [16] + 3
  lda LChars+3,Y          ; [19] + 4
  ora RChars+3,X          ; [23] + 4
  pha                     ; [27] + 3
  lda LChars+2,Y          ; [30] + 4
  ora RChars+2,X          ; [34] + 4
  pha                     ; [38] + 3
  lda LChars+1,Y          ; [41] + 4
  ora RChars+1,X          ; [45] + 4
  pha                     ; [49] + 3
  lda LChars+0,Y          ; [52] + 4
  ora RChars+0,X          ; [56] + 4
  pha                     ; [60] + 3

  ; Load Upper Score Digits
  lda SCORE+2             ; [63] + 3
  lsr                     ; [66] + 2
  lsr                     ; [68] + 2
  lsr                     ; [70] + 2
  lsr                     ; [72] + 2
  tax                     ; [74] + 2
  PLAY_MUSIC              ; [0] + 7
  
  ldy ScoreDigits,X       ; [7] + 4
  lda SCORE+2             ; [11] + 3
  and #%00001111          ; [14] + 2
  tax                     ; [16] + 2
  lda ScoreDigits,X       ; [18] + 4
  tax                     ; [22] + 2
  lda LChars+4,Y          ; [24] + 4
  ora RChars+4,X          ; [28] + 4
  pha                     ; [32] + 3
  lda LChars+3,Y          ; [35] + 4
  ora RChars+3,X          ; [39] + 4
  pha                     ; [43] + 3
  lda LChars+2,Y          ; [46] + 4
  ora RChars+2,X          ; [50] + 4
  pha                     ; [54] + 3
  lda LChars+1,Y          ; [57] + 4
  ora RChars+1,X          ; [61] + 4
  pha                     ; [65] + 3

  ; 3 Copies Medium
  lda #%00000110          ; [68] + 2
  sta NUSIZ1              ; [70] + 3
  PLAY_MUSIC              ; [73] + 7
  
  lda LChars+0,Y          ; [4] + 4
  ora RChars+0,X          ; [8] + 4
  pha                     ; [12] + 3
  jmp StartKernel2        ; [15] + 3

  ; Small Numbers Set - LHS Characters
LChars
L_55
  DC.B  %11000000
  DC.B  %00100000
  DC.B  %11000000
  DC.B  %10000000
L_22
  DC.B  %11100000
  DC.B  %10000000
  DC.B  %01000000
  DC.B  %00100000
L_33
  DC.B  %11000000
  DC.B  %00100000
  DC.B  %01100000
  DC.B  %00100000
L_99
  DC.B  %11000000
  DC.B  %00100000
  DC.B  %01100000
  DC.B  %10100000
L_00
  DC.B  %01000000
  DC.B  %10100000
  DC.B  %10100000
  DC.B  %10100000
L_88
  DC.B  %01000000
  DC.B  %10100000
  DC.B  %01000000
  DC.B  %10100000
L_77
  DC.B  %01000000
  DC.B  %01000000
  DC.B  %00100000
  DC.B  %00100000
L_11
  DC.B  %11100000
  DC.B  %01000000
  DC.B  %01000000
  DC.B  %11000000
L_66
  DC.B  %01000000
  DC.B  %10100000
  DC.B  %11000000
  DC.B  %10000000
  DC.B  %01100000
L_44
  DC.B  %00100000
  DC.B  %11100000
  DC.B  %10100000
  DC.B  %10000000
  DC.B  %10000000
EndLChars
  if (>LChars != >EndLChars)
    echo "WARNING: LChars crosses a page boundary!"
  endif

  ; Small Numbers Set - RHS Characters
RChars
R_55
  DC.B  %00001100
  DC.B  %00000010
  DC.B  %00001100
  DC.B  %00001000
R_22
  DC.B  %00001110
  DC.B  %00001000
  DC.B  %00000100
  DC.B  %00000010
R_33
  DC.B  %00001100
  DC.B  %00000010
  DC.B  %00000110
  DC.B  %00000010
R_99
  DC.B  %00001100
  DC.B  %00000010
  DC.B  %00000110 
  DC.B  %00001010
R_00
  DC.B  %00000100
  DC.B  %00001010
  DC.B  %00001010
  DC.B  %00001010
R_88
  DC.B  %00000100
  DC.B  %00001010
  DC.B  %00000100
  DC.B  %00001010
R_77
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00000010
  DC.B  %00000010
R_11
  DC.B  %00001110
  DC.B  %00000100
  DC.B  %00000100
  DC.B  %00001100
R_66
  DC.B  %00000100
  DC.B  %00001010
  DC.B  %00001100
  DC.B  %00001000
  DC.B  %00000110
R_44
  DC.B  %00000010
  DC.B  %00001110
  DC.B  %00001010
  DC.B  %00001000
  DC.B  %00001000
EndRChars
  if (>RChars != >EndRChars)
    echo "WARNING: RChars crosses a page boundary!"
  endif

  DC.B  "X7"
  echo "----",($FFF8 - *) , "bytes left (BANK 1 - MAIN GAME)"

  ORG     $0FF8
  RORG    $FFF8
  DC.B    0, 0
  DC.W    Init1, Init1, Init1
