; -----------------------------------------------------------------------------
; CHETIRY (GameBoyTM Version) - Atari 2600 - (C) Copyright 2011 - AtariAge
; BANK 2 - GAME KERNEL
; -----------------------------------------------------------------------------

  ; Self Modifying Kernel (42 bytes)
  MAC KERNEL
Modify
Col1
  lda #$FF                ; [17] + 2
  sta COLUPF              ; [19] + 3
Col2
  lda #$FF                ; [22] + 2
  sta COLUP0              ; [24] + 3
Col3
  lda #$FF                ; [27] + 2
  sta COLUP1              ; [29] + 3
Col4
  lda #$FF                ; [32] + 2
  php                     ; [34] + 3  = 37
  sta COLUBK              ; [37] + 3  >= 40 <= 48
Col5Y
  sty COLUPF              ; [40] + 3  > 42 < 51
Col6
  lda #$FF                ; [43] + 2
  sta COLUP0              ; [45] + 3  > 45 < 54
Col7
  lda #$FF                ; [48] + 2
  sta COLUP1              ; [50] + 3  >=48 <= 56
Col8
  lda #$FF                ; [53] + 2
  sta COLUBK              ; [55] + 3  > 50 < 59
Col9X
  stx COLUPF              ; [58] + 3  > 53 < 62
  stx VBLANK              ; [61] + 3  = 64
JPtr
  jmp Modify              ; [64] + 3
  ENDM

; BANK 2
  SEG     BANK2
  ORG     $1000
  RORG    $F000

  ; Space for DPC registers
  DS.B    128,0

StartGame
  ; Begin Game
  nop     $FFF8
  ; Entry Point
  jmp     Start
KernelExit
  ; Finish Kernel
  nop     $FFF8
  ; Begin Score Kernel
  jmp     ScoreKernel
  nop
  nop
  nop
RestartTitles
  ; Reset Music & SFX
  lda #0
  sta AUDV0
  sta AUDV1
  sta MUSIC
  sta MUSICPOS
  sta SFX

  ; Restart Title Screen
  jmp EndTitleFire

; -----------------------------------------------------------------------------
; TITLE SCREEN
; -----------------------------------------------------------------------------

Start
  CLEAN_START

TitleLoop
  START_VBLANK

  ; Randomize Blocks
  dec RANDOM

  ; Check Fire Button
  sta WSYNC
  lda INPT4
  bmi ResetTitleFire
  ; Debounce Fire
  lda DEBOUNCE
  and #%00100000 
  bne EndTitleFire
  ; Fire Button Pressed
  lda #%00100000
  sta DEBOUNCE
  ; Reset Sprites
  lda #0
  sta REFP0
  sta REFP1
  sta VDELP0
  sta VDELP1
  sta GRP0
  sta GRP1
  sta GRP0
  ; Begin Game
  jmp StartGame           ; [13] + 3
ResetTitleFire
  ; Clear Fire Button State
  lda DEBOUNCE            ; [6] + 3
  and #%11011111          ; [9] + 2
  sta DEBOUNCE            ; [11] + 3
EndTitleFire
  jmp ShowTitles

  DC.B  "K0"

  ALIGN 256

ShowTitles
  ; Set 3 Copies Close and Delay
  sta WSYNC               ; [0]
  lda #%00000011          ; [0] + 2
  sta VDELP0              ; [2] + 3
  sta VDELP1              ; [5] + 3
  sta NUSIZ0              ; [8] + 3
  sta NUSIZ1              ; [11] + 3
  ; Position Sprites (Fine)
  sta HMCLR               ; [14] + 3
  lda #%00010000          ; [17] + 2
  sta HMP1                ; [19] + 3
  lda #0                  ; [22] + 2
  sta GRP0                ; [24] + 3
  nop                     ; [27] + 2
  sta RESBL               ; [29] + 3    = 32
  sta GRP1                ; [32] + 3
  sta GRP0                ; [35] + 3
  ; Position Sprites (Coarse)
  sta RESP0               ; [38] + 3    = 41
  sta RESP1               ; [41] + 3    = 44

  ; Move Sprites
  sta WSYNC
  sta HMOVE

  ; Set Background
  lda #%00110001
  sta CTRLPF
  lda #0
  sta COLUPF
  sta COLUBK
  lda #$FF
  sta PF0
  lda #%11110000
  sta PF1
  lda #2
  sta ENABL

  ; Begin Screen
  START_FRAME

  ; Top Gap
  SKIP_LINES 30
  
  ; Colour Bars
  ldx #3
Loop1
  sta WSYNC
  lda ColBar1,X
  sta COLUBK
  dex
  bpl Loop1
  sta WSYNC
  lda #TCOL6
  sta COLUBK
  SKIP_LINES 2

  ; Load Chetiry Sprites
  ldy #(0*8)
  ldx #>ChetiryLogo0
  jsr ShowMessage
  SKIP_LINES 3

  ; Colour Bars
  ldx #3
Loop2
  sta WSYNC
  lda ColBar2,X
  sta COLUBK
  dex
  bpl Loop2
  ldx #3
Loop3
  sta WSYNC
  lda ColBar3,X
  sta COLUBK
  dex
  bpl Loop3
  sta WSYNC
  lda #TCOL4
  sta COLUBK

  ; Moon & Stars
  ldy #1*8
  ldx #>Moon0
  jsr ShowMessage

  ; Cathedral Spires
  ldy #2*8
  ldx #>Spires0
  jsr ShowMessage

  ; Colour Bars
  ldx #3
Loop4
  lda ColBar4,X
  sta COLUBK
  sta WSYNC
  dex
  bpl Loop4
  sta WSYNC
  lda #TCOL7
  sta COLUBK
  SKIP_LINES 2

  ; Copyright
  ldy #3*8
  ldx #>Copyright0
  jsr ShowMessage

  ; AtariAge
  sta WSYNC
  ldy #4*8
  ldx #>AtariAge0
  jsr ShowMessage

  ; Bottom Gap
  SKIP_LINES 4
  lda #0
  sta COLUBK

  START_OVERSCAN
  WAIT_OVERSCAN
  jmp TitleLoop

  DC.B  "K1"

; -----------------------------------------------------------------------------
; SCORE KERNEL
; -----------------------------------------------------------------------------

  ALIGN 256

ScoreKernel

  ; Wait for End of VBlank Region
WaitMainVblank
  PLAY_MUSIC
  lda INTIM
  bne WaitMainVblank
  sta VBLANK

  sta WSYNC               ; [0]         Line 0
  sta HMOVE               ; [0] + 3
  PLAY_MUSIC              ; [3] + 7
  nop                     ; [10] + 2
  lda #0                  ; [12] + 2
  sta PF1                 ; [14] + 3    <= 28
  lda SBUFF+4             ; [17] + 3
  sta GRP0                ; [20] + 3    < 38 (> 50)
  lda SBUFF+9             ; [23] + 3
  sta GRP1                ; [26] + 3    < 41 (>= 64)
  lda SBUFF+14            ; [29] + 3
  ldx SBUFF+19            ; [32] + 3
  ldy PFBUFF1             ; [35] + 3
  sta GRP0                ; [38] + 3    > 40 <= 43
  lda SBUFF+24            ; [41] + 3
  stx GRP0                ; [44] + 3    > 45 < 49
  sta GRP1                ; [47] + 3    >= 43 <= 50
  sty PF1                 ; [50] + 3    > 38 < 55
  lda SBUFF+29            ; [53] + 3
  sta GRP1                ; [56] + 3    > 53 < 62

  sta WSYNC               ; [0]         Line 1
  PLAY_MUSIC              ; [0] + 7
  SLEEP 5                 ; [7] + 5
  lda #0                  ; [12] + 2
  sta PF1                 ; [14] + 3    < = 28
  lda SBUFF+3             ; [17] + 3
  sta GRP0                ; [20] + 3    < 38 (> 50)
  lda SBUFF+8             ; [23] + 3
  sta GRP1                ; [26] + 3    <= 40 (>= 64)
  lda SBUFF+13            ; [29] + 3
  ldx SBUFF+18            ; [32] + 3
  ldy PFBUFF1             ; [35] + 3
  sta GRP0                ; [38] + 3    >= 40 < 43
  lda SBUFF+23            ; [41] + 3
  stx GRP0                ; [44] + 3    > 45 <= 48 
  sta GRP1                ; [47] + 3    > 41 < 51
  sty PF1                 ; [50] + 3    > 38 < 55
  lda SBUFF+28            ; [53] + 3
  sta GRP1                ; [56] + 3    > 53 < 62

  sta WSYNC               ; [0]         Line 2
  PLAY_MUSIC              ; [0] + 7
  SLEEP 8                 ; [7] + 8
  ldy #0                  ; [15] + 2
  sty PF1                 ; [17] + 3    < = 28
  lda SBUFF+2             ; [20] + 3
  sta GRP0                ; [23] + 3    < 38 (> 50)
  lda SBUFF+7             ; [26] + 3
  sta GRP1                ; [29] + 3    <= 40 (>= 64)
  lda SBUFF+12            ; [32] + 3
  ldx SBUFF+17            ; [35] + 3
  sta GRP0                ; [38] + 3    >= 40 < 43
  lda SBUFF+22            ; [41] + 3
  stx GRP0                ; [44] + 3    > 45 <= 48 
  sta GRP1                ; [47] + 3    > 41 < 51
  sty PF1                 ; [50] + 3    > 38 < 55
  lda SBUFF+27            ; [53] + 3
  sta GRP1                ; [56] + 3    > 53 < 62

  sta WSYNC               ; [0]         Line 3
  PLAY_MUSIC              ; [0] + 7
  SLEEP 5                 ; [7] + 5
  lda #0                  ; [12] + 2
  sta PF1                 ; [14] + 3    < = 28
  lda SBUFF+1             ; [17] + 3
  sta GRP0                ; [20] + 3    < 38 (> 50)
  lda SBUFF+6             ; [23] + 3
  sta GRP1                ; [26] + 3    <= 40 (>= 64)
  lda SBUFF+11            ; [29] + 3
  ldx SBUFF+16            ; [32] + 3
  ldy PFBUFF0             ; [35] + 3
  sta GRP0                ; [38] + 3    >= 40 < 43
  lda SBUFF+21            ; [41] + 3
  stx GRP0                ; [44] + 3    > 45 <= 48 
  sta GRP1                ; [47] + 3    > 41 < 51
  sty PF1                 ; [50] + 3    > 38 < 55
  lda SBUFF+26            ; [53] + 3
  sta GRP1                ; [56] + 3    > 53 < 62

  sta WSYNC               ; [0]         Line 4
  PLAY_MUSIC              ; [0] + 7
  SLEEP 5                 ; [7] + 5
  lda #0                  ; [12] + 2
  sta PF1                 ; [14] + 3    < = 28
  lda SBUFF+0             ; [17] + 3
  sta GRP0                ; [20] + 3    < 38 (> 50)
  lda SBUFF+5             ; [23] + 3
  sta GRP1                ; [26] + 3    <= 40 (>= 64)
  lda SBUFF+10            ; [29] + 3
  ldx SBUFF+15            ; [32] + 3
  ldy PFBUFF0             ; [35] + 3
  sta GRP0                ; [38] + 3    >= 40 < 43
  lda SBUFF+20            ; [41] + 3
  stx GRP0                ; [44] + 3    > 45 <= 48 
  sta GRP1                ; [47] + 3    > 41 < 51
  sty PF1                 ; [50] + 3    > 38 < 55
  lda SBUFF+25            ; [53] + 3
  sta GRP1                ; [56] + 3    > 53 < 62

  ; Score Cleanup
  ldy #0                  ; [59] + 2
  sty PF1                 ; [64] + 3
  sty COLUPF              ; [67] + 3
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sty GRP0                ; [7] + 3
  sty GRP1                ; [10] + 3
  sty COLUP0              ; [13] + 3
  sty COLUP1              ; [16] + 3

  jmp GridSetup           ; [19] + 3

EndScoreKernel
  if (>ScoreKernel != >EndScoreKernel)
    echo "WARNING: Score Kernel Crosses Page Boundary!"
  endif

  DC.B  "K2"

; -----------------------------------------------------------------------------
; GRID KERNEL SETUP
; -----------------------------------------------------------------------------

  ALIGN 256

GridSetup
  ; Kernel Setup
  ldy #$84                ; [22] + 2    STY
  sty KERNEL+17           ; [24] + 3
  iny                     ; [27] + 2    STA
  sty KERNEL+2            ; [29] + 3
  sty KERNEL+6            ; [32] + 3
  sty KERNEL+10           ; [35] + 3
  sty KERNEL+15           ; [38] + 3
  sty KERNEL+21           ; [41] + 3
  sty KERNEL+25           ; [44] + 3
  sty KERNEL+29           ; [47] + 3
  iny                     ; [50] + 2    STX
  sty KERNEL+31           ; [52] + 3
  sty KERNEL+33           ; [55] + 3
  ldy #COLUBK             ; [58] + 2    COLUBK
  sty KERNEL+16           ; [60] + 3
  sty KERNEL+30           ; [63] + 3
  ldy #COLUPF             ; [66] + 2    COLUPF
  sty KERNEL+3            ; [68] + 3
  sty KERNEL+18           ; [71] + 3
  sty KERNEL+32           ; [74] + 3

  PLAY_MUSIC              ; [1] + 7
  ldy #COLUP0             ; [8] + 2     COLUP0
  sty KERNEL+7            ; [10] + 3
  sty KERNEL+22           ; [13] + 3
  ldy #COLUP1             ; [16] + 2    COLUP1
  sty KERNEL+11           ; [18] + 3
  sty KERNEL+26           ; [21] + 3
  ldy #VBLANK             ; [24] + 2    VBLANK
  sty KERNEL+34           ; [26] + 3
  ldy #$4C                ; [29] + 2    JMP
  sty KERNEL+35           ; [31] + 3
  ldy #$A9                ; [34] + 2    LDA
  sty KERNEL+0            ; [36] + 3
  sty KERNEL+4            ; [39] + 3
  sty KERNEL+8            ; [42] + 3
  sty KERNEL+12           ; [45] + 3
  sty KERNEL+19           ; [48] + 3
  sty KERNEL+23           ; [51] + 3
  sty KERNEL+27           ; [54] + 3
  ldy #$08                ; [57] + 2    PHP
  sty KERNEL+14           ; [59] + 3

  ; Set Jump Pointer High
  ldy #>Line1             ; [62] + 2
  sty JPtr+2              ; [64] + 3

  ; Clear Sprite Positions
  sta HMCLR               ; [67] + 3

  ; Play Audio
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Set Sprite Data
  ldy #$FF                ; [7] + 2
  sty GRP0                ; [9] + 3
  sty GRP1                ; [12] + 3
  ldy #2                  ; [15] + 2
  sty ENABL               ; [17] + 3

  ; Set PF Data
  ldy #%00000110          ; [20] + 2
  sty PF2                 ; [22] + 3
  ldy #%01100000          ; [25] + 2
  sty PF0                 ; [27] + 3
  ldy #%00000110          ; [30] + 2
  sty PF1                 ; [32] + 3
  
  ; Set Stack Pointer
  ldx #VBLANK-1           ; [35] + 2

  ; Position Sprites
  sta.w RESP0             ; [37] + 4    = 41
  sta RESP1               ; [41] + 3    = 44
  
  ; Fine Position
  ldy #%00010000          ; [44] + 2
  sty HMP1                ; [46] + 3
  sty HMBL                ; [49] + 3

  ; Set Two Sprite Copies
  ldy #%00000010          ; [52] + 2
  sty NUSIZ0              ; [54] + 3
  sty NUSIZ1              ; [57] + 3
  
  ; Set Ball Position
  sta RESBL               ; [60] + 3    = 63
  txs                     ; [63] + 2

   ; Non-Reflected Playfield
  lda #0                  ; [65] + 2
  sta CTRLPF              ; [67] + 3
  
  ; Set Starting Line
  ldy #72                 ; [70] + 2
  sty TEMP                ; [72] + 3

  ; Move Sprites
  sta HMOVE               ; [75] + 3    > 74 < 4

  ; Update Music
  PLAY_MUSIC              ; [2] + 7
  
  ; Display Grid
  jmp StartGrid           ; [9] + 3     = 12

; -----------------------------------------------------------------------------
; FLASHING GRID LINE
; -----------------------------------------------------------------------------

FlashLine
  ; Set Flash Colour
  ldx #BCOL0              ; [43] + 2
  lda GRID+2,Y            ; [45] + 4
  asl                     ; [49] + 2
  bmi ShowFlashLine       ; [51] + 2/3
  bpl HideFlashLine       ; [53] + 3
ShowFlashLine
  ldx #BCOL8              ; [54] + 2
HideFlashLine

  ; Store Line
  sty TEMP                ; [56] + 3

  ; Enable Blank
  nop                     ; [59] + 2
  stx VBLANK              ; [61] + 3    = 64

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Store Kernel Colours
  stx TEMP2               ; [7] + 3
  stx Col1                ; [10] + 3 
  stx Col2                ; [13] + 3
  stx Col3                ; [16] + 3
  stx Col4                ; [19] + 3
  stx Col6                ; [22] + 3
  stx Col7                ; [25] + 3
  stx Col8                ; [28] + 3

  ; Unblank
  SLEEP 3                 ; [31] + 3
  sty VBLANK              ; [34] + 3    = 37

  ; Set Line Pointer
  lda #<Line1             ; [37] + 2
  sta JPtr+1              ; [39] + 3

  ; Reset Stack Pointer
  pla                     ; [42] + 4

  ; Copy Line Colour to Y
  txa                     ; [46] + 2
  tay                     ; [48] + 2
  
  ; Enable Blank
  SLEEP 11                ; [50] + 11
  sty VBLANK              ; [61] + 3    = 64
  
  ; Display Grid
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  SLEEP 4                 ; [7] + 4
  sty COLUBK              ; [11] + 3
  jmp Modify              ; [14] + 3

  DC.B  "K3"

; -----------------------------------------------------------------------------
; GRID KERNEL
; -----------------------------------------------------------------------------
  
  ALIGN 256

  ; Draw Grid Lines
Line1
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line2             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line2
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line3             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line3
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line4             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line4
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line5             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line5
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line6             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line6
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line7             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line7
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<Line8             ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3
Line8
  pla                     ; [67] + 4
  PLAY_MUSIC              ; [71] + 7
  lda #<GridLoop          ; [2] + 2
  sta JPtr+1              ; [4] + 3
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3

GridLoop
  ; Reset Colours
  ldx #BCOL0              ; [67] + 2
  stx COLUP0              ; [69] + 3
  stx COLUP1              ; [72] + 3
  PLAY_MUSIC              ; [75] + 7
  stx COLUBK              ; [6] + 3
  stx COLUPF              ; [9] + 3

StartGrid
  ; Calculate Current Line
  sec                     ; [12] + 2
  lda TEMP                ; [14] + 3
  sbc #4                  ; [17] + 2
  bmi EndGrid             ; [19] + 2/3
  tay                     ; [21] + 2

  ; Set First Line Pointer
  lda #<Line1             ; [23] + 2
  sta JPtr+1              ; [25] + 3

  ; Get First Grid Position
  lax GRID+0,Y            ; [28] + 4
  rol                     ; [32] + 2

  ; Disable Blank
  sty VBLANK              ; [34] + 3    = 37
  
  ; Check for flashing line
  bmi FlashingLine        ; [37] + 2/3

  ; Store Colours 0 & 1
  lda ColTable0,X         ; [39] + 4
  sta TEMP2               ; [43] + 3
  lda ColTable1,X         ; [46] + 4
  sta Col1                ; [50] + 3

  ; Get Second Grid Position
  ldx GRID+1,Y            ; [53] + 4
  lda ColTable0,X         ; [57] + 4

  ; Enable Blank
  sta VBLANK              ; [61] + 3    = 64

  ; Store Colour 2
  sta Col2                ; [64] + 3

  ; Store Line Counter
  sty TEMP                ; [67] + 3

  ; Reset Stack Pointer
  pla                     ; [70] + 4
  
  ; Continue Grid Kernel
  jmp ContGrid            ; [74] + 3

EndGrid
  ; Finish Kernel
  jmp FinishKernel        ; [22] + 3

FlashingLine
  ; Show Flashing Line
  jmp FlashLine           ; [40] + 3

  DC.B  "K4"
  
  ALIGN 256

ContGrid
  ; Get Color 3
  lda ColTable1,X         ; [1] + 4
  sta Col3                ; [5] + 3
  
  ; Store Colour 3 & 8
  txa                     ; [8] + 2
  ror                     ; [10] + 2
  tax                     ; [12] + 2
  lda ColTable2,X         ; [14] + 4
  sta Col8                ; [18] + 3

  ; Get Third Grid Position
  lax GRID+2,Y            ; [21] + 4
  rol                     ; [25] + 2
  lda ColTable0,X         ; [27] + 4
  
  ; Store Colour 4
  sta Col4                ; [31] + 3

  ; Disable Blank
  sty VBLANK              ; [34] + 3    = 37

  ; Get Colour 5
  lda ColTable1,X         ; [37] + 4
  
  ; Get Fourth Grid Position
  ldx GRID+3,Y            ; [41] + 4

  ; Store Colours 5 (in Y), 6 and 7
  tay                     ; [45] + 2
  lda ColTable0,X         ; [47] + 4
  sta Col6                ; [51] + 3
  lda ColTable1,X         ; [54] + 4
  sta Col7                ; [58] + 3
  
  ; Enable Blank
  sta VBLANK              ; [61] + 3    = 64

  ; Get Colour 9 (in X)
  txa                     ; [64] + 2
  ror                     ; [66] + 2
  tax                     ; [68] + 2
  lda ColTable2,X         ; [70] + 4
  tax                     ; [74] + 2
  PLAY_MUSIC              ; [0] + 7

  ; Display Grid (X = Colour 4, Y = Colour 9)
  lda TEMP2               ; [7] + 3
  sta.w COLUBK            ; [10] + 4
  jmp Modify              ; [14] + 3

; -----------------------------------------------------------------------------
; TITLE SCREEN KERNEL
; -----------------------------------------------------------------------------

  ; Message Pointers  (X = HiByte, Y = Message Offset)
ShowMessage
  sta WSYNC               ; [0]
  stx GPTR0+1             ; [0] + 3
  stx GPTR1+1             ; [3] + 3
  stx GPTR2+1             ; [6] + 3
  stx GPTR3+1             ; [9] + 3
  stx GPTR4+1             ; [12] + 3
  stx GPTR5+1             ; [15] + 3
  lda TitleMessages+0,Y   ; [18] + 4
  sta GPTR0               ; [22] + 3
  lda TitleMessages+1,Y   ; [25] + 4
  sta GPTR1               ; [29] + 3
  lda TitleMessages+2,Y   ; [32] + 4
  sta GPTR2               ; [36] + 3
  lda TitleMessages+3,Y   ; [39] + 4
  sta GPTR3               ; [43] + 3
  lda TitleMessages+4,Y   ; [46] + 4
  sta GPTR4               ; [50] + 3
  lda TitleMessages+5,Y   ; [53] + 4
  sta GPTR5               ; [57] + 3
  lda TitleMessages+6,Y   ; [60] + 4
  sta COLUP0              ; [64] + 3
  sta COLUP1              ; [67] + 3
  lda TitleMessages+7,Y   ; [70] + 4
  sta GLINE               ; [74] + 3
GameTextLoop
  ldy GLINE               ; [63] + 3
  lda (GPTR0),Y           ; [66] + 5
  sta WSYNC               ; [0]
  SLEEP 3                 ; [0] + 3
StartTextLoop
  sta GRP0                ; [3] + 3
  lda (GPTR1),Y           ; [6] + 5
  sta GRP1                ; [11] + 3    < 43
GameTextStart
  lda (GPTR2),Y           ; [14] + 5
  sta GRP0                ; [19] + 3    < 46
  lax (GPTR4),Y           ; [22] + 5
  lda (GPTR5),Y           ; [27] + 5
  sta TEMP                ; [32] + 3
  lda (GPTR3),Y           ; [35] + 5
  ldy TEMP                ; [40] + 3
  sta GRP1                ; [43] + 3    > 45 <= 48
  stx GRP0                ; [46] + 3    >= 48 < 51
  sty GRP1                ; [49] + 3    > 50 < 54
  sta GRP0                ; [52] + 3    > 53 <= 56
  dec GLINE               ; [55] + 5
  bpl GameTextLoop        ; [60] + 2/3
  ; Cleanup
  ldx #0                  ; [62] + 2
  stx GRP1                ; [64] + 3
  stx GRP0                ; [67] + 3
  stx GRP1                ; [70] + 3
  rts                     ; [73] + 6   = 3
EndShowMessage
  if (>ShowMessage != >EndShowMessage)
    echo "WARNING: ShowMessage crosses a page boundary!"
  endif

  DC.B  "K5"

  ALIGN 256

; -----------------------------------------------------------------------------
; FINISH KERNEL & PLAY SOUNDS/MUSIC
; -----------------------------------------------------------------------------

FinishKernel
  ; Clear Everything
  ldx #0                  ; [25] + 2
  stx COLUBK              ; [27] + 3
  stx COLUPF              ; [30] + 3
  stx COLUP0              ; [33] + 3
  stx COLUP1              ; [36] + 3
  stx GRP0                ; [39] + 3
  stx GRP1                ; [42] + 3
  stx PF0                 ; [45] + 3
  stx PF1                 ; [48] + 3
  stx PF2                 ; [51] + 3
  stx ENABL               ; [54] + 3

  ; Reset Stack
  dex                     ; [57] + 2
  txs                     ; [59] + 2

  ; Set Sound Table MSB
  ldx #>GSoundTab         ; [61] + 2
  stx SPTR+1              ; [63] + 3

  IF (PALMODE)
  ldx #7
PalSkipLoop
  sta WSYNC
  PLAY_MUSIC
  SLEEP 30
  dex
  bpl PalSkipLoop
  ENDIF

  ; Begin Overscan
  ldx #OVERDELAY          ; [66] + 2
  lda #2                  ; [68] + 2
  sta WSYNC               ; [0]
  sta VBLANK              ; [0] + 3
  PLAY_MUSIC              ; [3] + 7
  stx TIM64T              ; [10] + 3

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Play Sound Effects
  lax SFX                 ; [7] + 3
  beq EndGSounds          ; [10] + 2/3
  and #%00011111          ; [12] + 2
  tay                     ; [14] + 2
  dey                     ; [16] + 2
  bpl PlayGSound          ; [18] + 2/3
  lda #0                  ; [20] + 2
  sta SFX                 ; [22] + 3
  beq EndGSounds          ; [25] + 3
PlayGSound
  sty TEMP                ; [21] + 3
  txa                     ; [24] + 2
  and #%11100000          ; [26] + 2
  ora TEMP                ; [28] + 3
  sta SFX                 ; [31] + 3
  lsr                     ; [34] + 2
  lsr                     ; [36] + 2
  lsr                     ; [38] + 2
  lsr                     ; [40] + 2
  lsr                     ; [42] + 2
  tax                     ; [44] + 2
  lda GSoundTab,X         ; [46] + 4
  sta SPTR                ; [50] + 3
  lda (SPTR),Y            ; [53] + 5
  sta AUDF1               ; [58] + 3
  lda GSoundType,X        ; [61] + 4
  sta AUDC1               ; [65] + 3
  lda #8                  ; [68] + 2
EndGSounds
  sta AUDV1               ; [70] + 3

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  
  ; Tempo
  dec MUSIC               ; [7] + 5
  bpl SkipMusic           ; [12] + 2/3
  
  ; Reset Tempo
  lda #MUSICTEMPO         ; [14] + 2
  sta MUSIC               ; [16] + 3

  ; Fetch Music Position
  ldy MUSICPOS            ; [19] + 3
  inc MUSICPOS            ; [22] + 5

  ; Music Channel 0
  lda MusicData0,Y        ; [27] + 4
  beq Skip0B              ; [31] + 2/3
  cmp #REST               ; [33] + 2
  beq Rest0               ; [35] + 2/3
  sta DF7TOP              ; [37] + 3
  sta DF7LOW              ; [40] + 3
  lsr                     ; [43] + 3
  sta DF7BOT              ; [46] + 3
  lda #$FF                ; [49] + 2
  jmp Skip0A              ; [51] + 3
Rest0
  lda #$00                ; [38] + 2
Skip0A
  sta DF7HIGHMUSIC        ; [54] + 3
Skip0B

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7

  ; Music Channel 1
  lda MusicData1,Y        ; [7] + 4
  beq Skip1B              ; [11] + 2/3
  cmp #REST               ; [13] + 2
  beq Rest1               ; [15] + 2/3
  sta DF6TOP              ; [17] + 3
  sta DF6LOW              ; [20] + 3
  lsr                     ; [23] + 2
  sta DF6BOT              ; [25] + 3
  lda #$FF                ; [28] + 2
  jmp Skip1A              ; [30] + 3
Rest1
  lda #$00                ; [18] + 2
Skip1A
  sta DF6HIGHMUSIC        ; [33] + 3
Skip1B

  ; Music Channel 2
  lda MusicData2,Y        ; [36] + 4
  beq Skip2B              ; [40] + 2/3
  cmp #REST               ; [42] + 2
  beq Rest2               ; [44] + 2/3
  sta DF5TOP              ; [46] + 3
  sta DF5LOW              ; [49] + 3
  lsr                     ; [52] + 2
  sta DF5BOT              ; [54] + 3
  lda #$FF                ; [57] + 2
  jmp Skip2A              ; [59] + 3
Rest2
  lda #$00                ; [47] + 2
Skip2A
  sta DF5HIGHMUSIC        ; [62] + 3
Skip2B
SkipMusic

  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  jmp KernelExit          ; [7] + 3

  DC.B  "K6"

; -----------------------------------------------------------------------------
; GAME DATA
; -----------------------------------------------------------------------------

  ; Music Data
  ALIGN 256
  MUSICDATA0
  ALIGN 256
  MUSICDATA1
  ALIGN 256
  MUSICDATA2

  ; Sound Effects (Lower = Higher Priority)
  ; 0 = Win Sound
  ; 1 = Line 1
  ; 2 = Line 2
  ; 3 = Line 3
  ; 4 = Line 4
  ; 5 = Settle
  ; 6 = Rotate
  ; 7 = Fail Sound
  ALIGN 256
GSoundTab
  DC.B  <GWinSnd, <GLine1Snd, <GLine2Snd, <GLine3Snd
  DC.B  <GLine4Snd, <GSettleSnd, <GRotateSnd, <GFailSnd
GSoundType
  DC.B  6, 12, 12, 12, 12, 2, 12, 7
; GSoundLen
; DC.B  30, 30, 30, 30, 30, 3, 7, 31
GLine1Snd
  DC.B  31, 31, 31, 15, 15, 15, 31, 31, 31, 15, 15, 15, 31, 31, 31
  DC.B  20, 20, 20, 6, 6, 6, 20, 20, 20, 6, 6, 6, 20, 20, 20
GLine2Snd
  DC.B  31, 31, 31, 7, 7, 7, 31, 31, 31, 7, 7, 7, 31, 31, 31
  DC.B  20, 20, 20, 6, 6, 6, 20, 20, 20, 6, 6, 6, 20, 20, 20
GLine3Snd
  DC.B  31, 31, 31, 3, 3, 3, 31, 31, 31, 3, 3, 3, 31, 31, 31
  DC.B  20, 20, 20, 6, 6, 6, 20, 20, 20, 6, 6, 6, 20, 20, 20
GLine4Snd
  DC.B  31, 31, 31, 0, 0, 0, 31, 31, 31, 1, 1, 1, 31, 31, 31
  DC.B  20, 20, 20, 2, 2, 2, 20, 20, 20, 3, 3, 3, 20, 20, 20
GSettleSnd
  DC.B  10, 8, 6, 4
GRotateSnd
  DC.B  8, 10, 12, 14, 18, 16, 20, 18
GWinSnd
  DC.B  0, 1, 1, 1, 3, 3, 3, 2, 2, 2, 4, 4, 4, 3, 3, 3
  DC.B  5, 5, 5, 2, 2, 2, 7, 7, 7, 4, 4, 4, 8, 8, 8
GFailSnd
  DC.B  30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30
  DC.B  15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15
EndGSoundTab
  if (>GSoundTab != >EndGSoundTab)
    echo "WARNING: Game Sound Table Crosses Page Boundary!"
  endif

  ; Title Data (Part 2)
  SPRITES2
  DC.B  "APRIL1"
    
  ; Color Table 0 (256 Bytes)
  ; MASK = %00000111
  ALIGN 256
ColTable0
  REPEAT 32
  DC.B  BCOL0, BCOL1, BCOL2, BCOL3, BCOL4, BCOL5, BCOL6, BCOL7
  REPEND

  ; Color Table 1 (256 Bytes)
  ; MASK = %00111000
  ALIGN 256
ColTable1
  REPEAT 4
  DS.B  8, BCOL0
  DS.B  8, BCOL1
  DS.B  8, BCOL2
  DS.B  8, BCOL3
  DS.B  8, BCOL4
  DS.B  8, BCOL5
  DS.B  8, BCOL6
  DS.B  8, BCOL7
  REPEND

  ; Color Table 2 (256 Bytes)
  ; MASK = %11100000
  ALIGN 256
ColTable2
  DS.B  32, BCOL0
  DS.B  32, BCOL1
  DS.B  32, BCOL2
  DS.B  32, BCOL3
  DS.B  32, BCOL4
  DS.B  32, BCOL5
  DS.B  32, BCOL6
  DS.B  32, BCOL7

  ALIGN 256
  
  ; Title Screen Data (Part 1)
  SPRITES1

TitleMessages
  ; Chetiry Logo
  DC.B  <ChetiryLogo0, <ChetiryLogo1, <ChetiryLogo2
  DC.B  <ChetiryLogo3, <ChetiryLogo4, <ChetiryLogo5
  DC.B  TCOL0, 19
  ; Moon & Stars
  DC.B  <Moon0, <Moon1, <Moon2
  DC.B  <Moon3, <Moon4, <Moon5
  DC.B  TCOL7, 10
  ; Cathedral Spires
  DC.B  <Spires0, <Spires1, <Spires2
  DC.B  <Spires3, <Spires4, <Spires5
  DC.B  TCOL0, 34
  ; Copyright Message
  DC.B  <Copyright0, <Copyright1, <Copyright2
  DC.B  <Copyright3, <Copyright4, <Copyright5
  DC.B  TCOL0, 6
  ; AtariAge Logo
  DC.B  <AtariAge0, <AtariAge1, <AtariAge2
  DC.B  <AtariAge3, <AtariAge4, <AtariAge5
  DC.B  TCOL0, 8
EndTitleMessages
  if (>TitleMessages != >EndTitleMessages)
    echo "WARNING: TitleMessages crosses a page boundary!"
  endif

  ; Colour Bars
ColBar1
  DC.B  TCOL4, TCOL6, TCOL4, TCOL4
ColBar2
  DC.B  TCOL0, TCOL0, TCOL6, TCOL0
ColBar3
  DC.B  TCOL6, TCOL4, TCOL6, TCOL6
ColBar4
  DC.B  TCOL0, TCOL0, TCOL4, TCOL0
EndColBars
  if (>ColBar1 != >EndColBars)
    echo "WARNING: ColBars crosses a page boundary!"
  endif

  DC.B  "CCCP"

  ALIGN 256
  
  ; Title Screen Data (Part 3)
  SPRITES3

  DC.B  "Chetiry - GameBoy(tm) Demo Version - "
  DC.B  "(C) Copyright 2011 - "
  DC.B  "Distributed exclusively by AtariAge.com"

  echo "----",($FFF8 - *) , "bytes left (BANK 2 - KERNEL)"

  ORG     $1FF8
  RORG    $FFF8
  DC.B    0, 0
  DC.W    Start, Start, Start

