; -----------------------------------------------------------------------------
; CHETIRY (GameBoyTM Version) - Atari 2600 - (C) Copyright 2011 - AtariAge
; Programming - Chris Walton (cd-w)
; Kernel      - Zach Matley (zach)
; Artwork     - Nathun Strum
; -----------------------------------------------------------------------------

  PROCESSOR 6502

  INCLUDE "vcs.h"
  INCLUDE "macro.h"

  INCLUDE "dpc.h"
  INCLUDE "gbdata.h"

; -------------------------- GAME MACROS --------------------------------------

  ; Play DPC Music
  MAC PLAY_MUSIC
  lda $1006               ; [0] + 4
  sta AUDV0               ; [4] + 3
  ENDM

  ; Start Vertical Blank Macro
  MAC START_VBLANK
  lda #%00001110
.SyncLoop
  sta WSYNC               ; [0]
  sta VSYNC               ; [0] + 3
  lsr                     ; [3] + 2
  bne .SyncLoop           ; [5] + 2/3
  lda #VBLANKDELAY        ; [7] + 2
  sta TIM64T              ; [9] + 3
  ENDM

  ; Start Vertical Blank Macro (Music Version)
  MAC START_VBLANK_MUSIC
  lda #2                  ; VSYNC enable
  sta WSYNC               ; [0]
  sta VSYNC               ; [0] + 3
  PLAY_MUSIC              ; [3] + 7
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  ldx #VBLANKDELAY        ; [7] + 2
  lda #0                  ; [9] + 2
  sta WSYNC               ; [0]
  sta VSYNC               ; [0] + 3
  PLAY_MUSIC              ; [3] + 7
  stx TIM64T              ; [10] + 3
  ENDM

  ; Start Frame Macro
  MAC START_FRAME
.WaitVblank
  lda INTIM
  bne .WaitVblank
  sta VBLANK
  sta WSYNC               ; [0]
  lda #SCREENDELAY        ; [0] + 2
  sta TIM64T              ; [2] + 3
  ENDM

  ; Start Frame Macro (Music Version)
  MAC START_FRAME_MUSIC
  ldx #SCREENDELAY
.WaitVblank
  PLAY_MUSIC
  lda INTIM
  bne .WaitVblank
  sta VBLANK
  sta WSYNC               ; [0]
  PLAY_MUSIC              ; [0] + 7
  stx TIM64T              ; [7] + 3
  ENDM

  ; Start Overscan Macro
  MAC START_OVERSCAN
.WaitFrame
  lda INTIM
  bne .WaitFrame
  lda #2
  sta WSYNC               ; [0]
  sta VBLANK              ; [0] + 3
  lda #OVERDELAY          ; [3] + 2
  sta TIM64T              ; [5] + 3
  ENDM

  ; Start Overscan Macro (Music Version)
  MAC START_OVERSCAN_MUSIC
  ldx #OVERDELAY
.WaitFrame
  PLAY_MUSIC
  lda INTIM
  bne .WaitFrame
  lda #2
  sta WSYNC               ; [0]
  sta VBLANK              ; [0] + 3
  PLAY_MUSIC              ; [3] + 7
  stx TIM64T              ; [10] + 3
  ENDM

  ; Wait For Overscan End Macro
  MAC WAIT_OVERSCAN
.WaitOverscan
  lda INTIM
  bne .WaitOverscan
  ENDM
  
  ; Wait For Overscan End Macro (Music Version)
  MAC WAIT_OVERSCAN_MUSIC
.WaitOverscan
  PLAY_MUSIC
  lda INTIM
  bne .WaitOverscan
  ENDM

  ; Skip Lines Macro
  MAC SKIP_LINES
.LINES  SET {1}
  ldx #.LINES
.SkipLoop
  sta WSYNC
  dex
  bpl .SkipLoop
  ENDM

; -------------------------- GAME CONSTANTS -----------------------------------

; Game Constants
NO_ILLEGAL_OPCODES        = 0           ; Allow Undocumented Opcodes
PALMODE                   = 0           ; PAL/NTSC (0 = NTSC)
RANDOMSEED                = 15          ; Initial Random Number

; Time Constants
  IF (PALMODE)
VBLANKDELAY               = 62          ; PAL Frame Timings
SCREENDELAY               = 255
OVERDELAY                 = 50
HDELAY                    = 5           ; Horizontal Movement Delay (max 7)
VDELAY                    = (3 << 3)    ; Vertical Movement Delay (max 3)
FLASHTIME                 = 26          ; Flash Timer (max 31)
GAMEOVERTIME              = 26          ; Gameover Timer (max 31)
MUSICTEMPO                = 5           ; Default Music Tempo
  ELSE
VBLANKDELAY               = 44          ; NTSC Frame Timings
SCREENDELAY               = 246
OVERDELAY                 = 18
HDELAY                    = 6           ; Horizontal Movement Delay (max 7)
VDELAY                    = (3 << 3)    ; Vertical Movement Delay (max 3)
FLASHTIME                 = 31          ; Flash Timer (max 31)
GAMEOVERTIME              = 31          ; Gameover Timer (max 31)
MUSICTEMPO                = 6           ; Default Music Tempo
  ENDIF

; Title Screen Colours
  IF (PALMODE)            ; PAL
TCOL0     = $30
TCOL1     = $32
TCOL2     = $34
TCOL3     = $36
TCOL4     = $38
TCOL5     = $3A
TCOL6     = $3C
TCOL7     = $3E
  ELSE                    ; NTSC
TCOL0     = $D0
TCOL1     = $D2
TCOL2     = $D4
TCOL3     = $D6
TCOL4     = $D8
TCOL5     = $DA
TCOL6     = $DC
TCOL7     = $DE
  ENDIF

; Block Colours (Bit 1 Must Be Set)
  IF (PALMODE)            ; PAL
BCOL0     =  $36
BCOL1     =  $3E
BCOL2     =  $32
BCOL3     =  $3A
BCOL4     =  $3E
BCOL5     =  $32
BCOL6     =  $3A
BCOL7     =  $3E
BCOL8     =  $02
  ELSE                    ; NTSC
BCOL0     =  $D6
BCOL1     =  $DE
BCOL2     =  $D2
BCOL3     =  $DA
BCOL4     =  $DE
BCOL5     =  $D2
BCOL6     =  $DA
BCOL7     =  $DE
BCOL8     =  $02
  ENDIF

  SEG.U   VARS
  ORG     $80

; -------------------------- GLOBAL VARIABLES ---------------------------------

RANDOM    DS.B 1          ; Random Seed

LEVEL     DS.B 1          ; 0-4 = Current Level
SCORE     DS.B 3          ; Current Score (6-Digit BCD)

MUSIC     DS.B 1          ; 0-4 = Tempo
MUSICPOS  DS.B 1          ; 0-7 = Music Position
SFX       DS.B 1          ; 0-4 = Sound Position, 5-7 = Sound Effect

DEBOUNCE  DS.B 1          ; 0-2 = L/R Delay, 3-4 = D Delay, 5 = Fire, 6-7 = L/R
TEMP      DS.B 1          ; All-purpose Temp
TEMP2     DS.B 1          ; Colour 0 Temp

; -------------------------- GAME VARIABLES -----------------------------------

LINES     DS.B 2          ; Line Count/Timer (3 Digit BCD)
                          ; 4-5 = Unused
                          ; 6-7 = Mode (00 = Normal, 01 = Settle,
                          ;             10 = Flash, 11 = Game Over)
XPOS      DS.B 1          ; 0-3 = XPos, 4-6 = Type, 7 = Joystick Up
YPOS      DS.B 1          ; 0-4 = YPos, 5-6 = Rot, 7 = Hard Drop
TIMER     DS.B 1          ; 0-4 = Drop Timer, 5-7 = Next Piece (& GameOver Data)

; Chetiry Grid (10x18): (8T)10|(8)32|(9T)54|(9)76
GRID      DS.B 72

; Display Remaining RAM
  echo "----",($DA - *) , "bytes left (RAM)"

; -------------------------- TEMPORARY VARIABLES ------------------------------

; Self Modifying Kernel
KERNEL    EQU $DA         ; Chetiry Kernel (37 bytes)
Modify    EQU KERNEL+0    ; Entry Point
Col1      EQU KERNEL+1    ; Colour Values
Col2      EQU KERNEL+5
Col3      EQU KERNEL+9
Col4      EQU KERNEL+13
Col6      EQU KERNEL+20
Col7      EQU KERNEL+24
Col8      EQU KERNEL+28
JPtr      EQU KERNEL+35   ; Return Address

; Score Kernel
SBUFF     EQU $DA         ; 30 bytes (6*5)
PFBUFF0   EQU $F8
PFBUFF1   EQU $F9

; Game Text Kernel
GPTR0     EQU $DA
GPTR1     EQU $DC
GPTR2     EQU $DE
GPTR3     EQU $E0
GPTR4     EQU $E2
GPTR5     EQU $E4
GLINE     EQU $E6

; Temp Game Values
TEMPX     EQU $DA
TEMPX2    EQU $DB
TEMPY     EQU $DC
TEMPROT   EQU $DD
TEMPROT2  EQU $DE
TEMPVAL   EQU $DF
VALUE1    EQU $E0
VALUE2    EQU $E1
VALUE3    EQU $E2
VALUE4    EQU $E3
SPTR      EQU $E4
SPTR2     EQU $E6

; -----------------------------------------------------------------------------
; BANKS
; -----------------------------------------------------------------------------

  INCLUDE bank1.asm
  INCLUDE bank2.asm

  ; Bank 3 (DPC DATA)
  SEG     DATA
  ORG     $2000
  RORG    $F000
  DS.B    $800,$CC       ; Unused
  DS.B    $FF,$00        ; Unused

; -----------------------------------------------------------------------------

